import glob,os,filecmp,time,schedule,datetime, sys, getopt, requests
from shutil import copytree
# msvcrt is a windows specific native module
import msvcrt
import time
from dlab import dlabbehavior as db
import pandas as pd

from slack import WebClient
sc = WebClient(token=os.environ['SLACK_BEHAVIOR_BOT_TOKEN'])



# asks whether a key has been acquired
def kbfunc():
    #this is boolean for whether the keyboard has bene hit
    x = msvcrt.kbhit()
    if x:
        #getch acquires the character encoded in binary ASCII
        ret = msvcrt.getch()
    else:
        ret = False
    return ret


def job():
    folders = [r'C:\Users\denma\Desktop\cheetah_or_elephant',
            r'C:\Users\denma\Desktop\bonsai_levertask']
    print('syncing C: to backups at '+datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
    for task in folders:
        task_animals = glob.glob(os.path.join(task,'data')+r'\*')
        for folder in task_animals:
            make_plots=False #used later to determine if we make plots for this animal today
            #if an animal doesn't exist within a task data folder on E: or //DENMANLAB/s1, then make it
            try:
                if not os.path.isdir(folder.replace('C:\\Users\denma\Desktop','E:')): os.mkdir(folder.replace('C:\\Users\denma\Desktop','E:'))
            except:pass
            try:
                if not os.path.isdir(folder.replace('C:\\Users\denma\Desktop','//DENMANLAB/s1/behavior/').replace('\\','/')):os.mkdir(folder.replace('C:\\Users\denma\Desktop','//DENMANLAB/s1/behavior/').replace('\\','/'))
            except:pass 

            #this generates a comparison of the sessions within an animal's folder
            C_D = filecmp.dircmp(folder,folder.replace('C:\\Users\denma\Desktop','D:'))
            C_E = filecmp.dircmp(folder,folder.replace('C:\\Users\denma\Desktop','E:'))
            C_S1 = filecmp.dircmp(folder,folder.replace('C:\\Users\denma\Desktop','//DENMANLAB/s1/behavior/').replace('\\','/'))                        

            #if there are any sessions that are only on the C: drive, then it copies to E: and tries the same to //DENMANLAB/s1 
            if len(C_D.left_only)>0:
                for session in C_D.left_only:
                    if len(C_E.left_only)>0:
                        if session in C_E.left_only:
                            print('syncing C: '+session+' to E: at '+datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
                            #makenwb
                            db.make_lever_NWB(os.path.join(folder,session)) 
                            make_plots=True
                            #copy it all, including the nwb we just made
                            copytree(os.path.join(folder,session),os.path.join(folder.replace('C:\\Users\denma\Desktop','E:'),session))
            if len(C_S1.left_only)>0:
                for session in C_S1.left_only:
                    try:
                        print('syncing C '+session+' to DENMANLAB/s1 at '+datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
                        copytree(os.path.join(folder,session),folder.replace('C:\\Users\denma\Desktop','//DENMANLAB/s1/behavior').replace('\\','/')+r'/'+session)
                    except:pass
           
            #make today's figures
            if make_plots:
                animal = os.path.basename(folder)
                datestring=datetime.datetime.now().strftime("%Y_%m_%d").replace("_0","_")
                if task == folders[0]:#cheetah_or_elephant
                    pass # fr nw
                if task == folders[1]:#lever
                    #generate figures
                    fig,ax = db.generate_session_lever(animal,datestring,session='combine',return_='fig')
                    df = db.generate_session_lever(animal,datestring,session='combine',return_='df')
                    print(df)
                    date_folder = glob.glob(os.path.join(folder,animal+'_'+datestring+'*'))[-1] 
                    if not os.path.exists(os.path.join(date_folder,'figs')):os.makedirs(os.path.join(date_folder,'figs'))
                    fig.savefig(os.path.join(date_folder,'figs','summary.png'))
                    fig.savefig(os.path.join(date_folder,'figs','summary.eps'),format='eps')

                    #go over previous sessions and make across-session figures 
                    #first, get the strings for the previous ten days. do gymnastics to formats date for bonsai task outputs. 
                    # today = datestring.split('_')[2]
                    # tomonth =  datestring.split('_')[1]
                    # toyear = datestring.split('_')[0]
                    # days = []
                    # mdays = [31,31,28,31,30,31,30,31,31,30,31,30]
                    # mmonths = [12,1,2,3,4,5,6,7,8,9,10,11]
                    # for day_minus in range(10):
                    #     day_ = int(today)-day_minus
                    #     if day_ < 1:
                    #         last_month = int(tomonth)-1
                    #         day_ = mdays[last_month]+day_
                    #         if mmonths[last_month] > int(tomonth):
                    #             lastyear = str(int(toyear) - 1)
                    #             y_m_d = lastyear+'_'+str(mmonths[last_month])+'_'+str(day_)
                    #         else: y_m_d = toyear+'_'+str(mmonths[last_month])+'_'+str(day_)
                    #     else:
                    #         y_m_d = toyear+'_'+tomonth+'_'+str(day_)
                    #     days.extend([y_m_d])
        
                    # dfs = [db.generate_session(folder,datestring,session='combine',return_='df') ]
                    # df = pd.concat(dfs,ignore_index=True)
                    # fig = db.across_session_plots_lever(df)
                    # if not os.path.exists(os.path.join(folder,'figs')):os.makedirs(os.path.join(folder,'figs'))
                    # fig.savefig(os.path.join(folder,'figs','recent_sessions.png'))
                    # fig.savefig(os.path.join(folder,'figs','recent_sessions.eps'),format='eps')

                    #send figures to slack     
                    # message = 'behavior updates for '+animal+' on '+datestring
                    sc.chat_postMessage( channel="#behavior_plot_bot", text=message)
                    response = sc.files_upload(channels="#behavior_plot_bot",
                        file=os.path.join(date_folder,'figs','summary.png'),
                        title='summary '+animal+' on '+datestring)
                    # response = sc.files_upload(channels="#behavior_plot_bot",
                    #     file=os.path.join(folder,'figs','recent_sessions.png'),
                    #     title='summary '+folder+' on '+datestring)

    print('done syncing C: to backups at '+datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"))    
       

    return
#run at startup
print('backing up C to DENMANLAB')
job()

#schedule to recur every day at a certain time.
schedule.every().day.at("01:00").do(job)

while True:
    schedule.run_pending()
    time.sleep(60) # wait one minute
    #if we got a keyboard hit
    x = kbfunc()
    if x != False and x.decode() == 'u':
        #we got the key!
        #because x is a binary, we need to decode to string
        #use the decode() which is part of the binary object
        #by default, decodes via utf8
        #concatenation auto adds a space in between
        job()


#---------------------EXTRA STUFF FROM DEV------------------------------------------------------

def copytree2(source,dest):
    os.mkdir(dest)
    dest_dir = os.path.join(dest,os.path.basename(source))
    copytree(source,dest_dir)


# folders = [r'C:\Users\denma\Desktop\cheetah_or_elephant',
#           r'C:\Users\denma\Desktop\bonsai_levertask']


# for folder in folders:
#     C_D = filecmp.dircmp(os.path.join(folder,'data'),os.path.join('D:\\',os.path.basename(folder),'data'))
#     # C_S1 = filecmp.dircmp(os.path.join(folder,'data'),os.path.join('DENMANLAB','s1','behavior',os.path.basename(folder),'data'))
#     if len(C_D.left_only)>0:
#         for mouse in C_D.left_only:
#             copytree2(os.path.join(folder,'data',mouse),os.path.join('D:\\',os.path.basename(folder),'data',mouse))


# for sess in C_D.left_only:
#     copytree2(folder+r'/'+sess,os.path.join('D:\\','cheetah_or_elephant','data','c63',sess))


import numpy as np
import scipy, os
from scipy.signal import butter,lfilter
from scipy.ndimage.filters import gaussian_filter1d
import matplotlib.pyplot as plt
from matplotlib.pyplot import mlab
import xml.etree.ElementTree
samplingRate=30000.
#=================================================================================================
#------------operations on continuous traces-------------------------------------
#=================================================================================================

npix_p3_reference_channels = np.array([ 36,  75, 112, 151, 188, 227, 264, 303, 340, 379])
npix_p2_reference_channels = np.array([1,18,33,50,65,82,97,114,99])
skip_channels = npix_p3_reference_channels #default to phase 3 reference channels
def get_chunk(mm,start,end,channels,sampling_rate=30000,remove_offset=False):
	chunk = mm[int(start*sampling_rate*int(channels)):int(np.floor(end*sampling_rate*(int(channels))))]
	chunk = np.reshape(chunk,(int(channels),-1),order='F')  * 0.195
	# print(np.shape(chunk))
	if remove_offset:
		for i in np.arange(channels):
			chunk[i,:] = chunk[i,:] - np.mean(chunk[i,:])
	return chunk

#filter a bit of continuous data. uses butterworth filter.
def filterTrace(trace, low, high, sampleHz, order):
    low = float(low)
    high = float(high)
    nyq = 0.5 * sampleHz
    low = low / nyq
    high = high / nyq
    b, a = butter(order, [low, high], btype='band')
    filtered = lfilter(b, a, trace)
    return filtered
    
#developmental filter version. not used.
def filterTrace_hard(trace, low, high, sampleHz, order):
    low = float(low)
    high = float(high)
    nyq = 0.5 * sampleHz
    low = low / nyq
    high = high / nyq
    scipy.signal.band_stop_obj()
    b, a = butter(order, [low, high], btype='band')
    filtered = lfilter(b, a, trace)
    scipy.signal.lfilter()
    return filtered

#wrapper for filtering continous data of different forms.
#data can be a single continuous trace, a dictionary containing a key called 'data' whose value is a continous trace, or a dictionary of traces, or a dicit
def filtr(data,low, high, sampleHz, order):
    if type(data) is dict:
        if 'data' in data.keys():
            return filterTrace(data['data'],low, high, sampleHz, order)
        else:
            out = {}
            for i,key in enumerate(data.keys()):
                out[key] = data[key]
                out[key]['data']= filterTrace(data[key]['data'],low, high, sampleHz, order)
            return out
    else:
        return filterTrace(data,low, high, sampleHz, order)

#notch filter a continous trace by filtering in a narrow range and subtracting that from the input trace.    
def notch(data,freq, sampleHz):
    order = 1
    low = freq-2
    high = freq +2
    if type(data) is dict:
        if 'data' in data.keys():
            return data['data'] - filterTrace(data['data'],low, high, sampleHz, order)
        else:
            out = {}
            for i,key in enumerate(data.keys()):
                out[key] = data[key]
                out[key]['data']= data[key]['data'] - filterTrace(data[key]['data'],low, high, sampleHz, order)
            return out
    else:
        return data - filterTrace(data,low, high, sampleHz, order)

#average a continuous trace around a set of timestamps 
def average_trials(data,timestamps,window,sampleFreq=25000.):
    alltrials = np.zeros((len(timestamps),window*sampleFreq))
    average = np.zeros(window*sampleFreq)
    skipped = 0
    for i,onset in enumerate(timestamps):
        average += data[onset:onset+window*sampleFreq]#-np.mean(data[onset:onset-500])
        alltrials[i,:] = data[onset:onset+window*sampleFreq]#-np.mean(data[onset:onset-500])
        
#        if np.max(np.abs(data[onset:onset+window*sampleFreq]-np.mean(data[onset:onset+5000]))) < 40000000.0:
#            average += data[onset:onset+window*sampleFreq]-np.mean(data[onset:onset+5000])
#            alltrials[i,:] = data[onset:onset+window*sampleFreq]-np.mean(data[onset:onset+5000])
#        else:
#            skipped += 1
#            print 'skipped trial: '+str(i+1)
#            alltrials[i,:] = data[onset:onset+window*sampleFreq]-np.mean(data[onset:onset+5000])
    return alltrials,average/float(len(timestamps-skipped))   

#average all continuous traces in an array around a set of timestamps 
def average_trials_array(data,timestamps,window,output='avg'):
    avgs = {}
    alltrials={}
    for i,key in enumerate(data.keys()):
    	if 'data' in data[key].keys():
	        avgs[key]={}
	        alltrials[key]={}
	        alltrials[key]['data'],avgs[key]['data'] = average_trials(data[key]['data'],timestamps,window)
    if output == 'trials':
        return alltrials
    if output == 'both':
        return (alltrials,avgs)
    if output=='avg':
        return avgs
	

#note: this CSD code does not work! -dan
def CSD_1D(data,channelmap=[],prefix='100_CH',point=1000):
    if channelmap == []:
        channelmap = data.keys()
    elec_pos = []
    pots=[]    
    for i,key in enumerate(channelmap[0]):
        key = prefix+str(key).replace(prefix,'')
        pots.append([data[key]['data'][point]])
        elec_pos.append([(i+i)/2])
    pots=np.array(pots)
    elec_pos=np.array(elec_pos)
    params = {
        'xmin': 0,
        'xmax': 65.0,
        'source_type': 'step',
        'n_sources': 64,
        'sigma': 0.1
        }
    k = KCSD(elec_pos, pots, params)
    k.estimate_pots()
    k.estimate_csd()
    k.plot_all()

#note: this CSD code does not work! -dan    
def CSD_1D_time(data,channelmap=[],prefix='100_CH',point=1000):
    if channelmap == []:
        channelmap = data.keys()
    numPoints = len(data[data.keys()[0]]['data'])
    out_csd = np.zeros((len(data.keys()),numPoints))
    out_pots = np.zeros((len(data.keys()),numPoints))
    for point in range(numPoints):
        print(point)
        elec_pos = []
        pots=[]    
        for i,key in enumerate(channelmap[0]):
            key = prefix+str(key).replace(prefix,'')
            pots.append([data[key]['data'][point]])
            elec_pos.append([i+i])
        pots=np.array(pots)
        elec_pos=np.array(elec_pos)
        params = {
            'xmin': 0,
            'xmax': 130.0,
            'source_type': 'step',
            'n_sources': 128,
            'sigma': 0.2,
            }
        k = KCSD(elec_pos, pots, params)
        k.estimate_pots()
        k.estimate_csd()
        out_csd[0:np.shape(k.solver.estimated_csd)[0],point]= k.solver.estimated_csd[:,0]
        out_pots[0:np.shape(k.solver.estimated_pots)[0],point]= k.solver.estimated_pots[:,0]
    return out_csd,out_pots
        #k.plot_all()

def etree_to_dict(t):
    d = {t.tag : list(map(etree_to_dict, t.getchildren()))}
    d.update(('@' + k, v) for k, v in t.attrib.items())
    d['text'] = t.text
    return d

def get_channel_count(path,from_channel_map = True,from_templates=False):
	d = etree_to_dict(xml.etree.ElementTree.parse(os.path.join(path,'settings.xml')).getroot())
	chs =0
	if from_templates:
		return np.load(open(os.path.join(path,'templates.npy'))).shape[-1]
	if d['SETTINGS'][1]['SIGNALCHAIN'][0]['@name'] == 'Sources/Neuropix':
		for info in d['SETTINGS'][1]['SIGNALCHAIN'][0]['PROCESSOR'][:385]:
			if 'CHANNEL' in info.keys():
				if info['CHANNEL'][0]['@record'] == '1':
					chs +=1
		return chs
	if d['SETTINGS'][1]['SIGNALCHAIN'][0]['@name'] == 'Sources/Rhythm FPGA':
		if from_channel_map:
			for nm in d['SETTINGS'][1]['SIGNALCHAIN']:
				name = nm['@name']
				if name == 'Filters/Channel Map':
					#chs = np.shape(d['SETTINGS'][1]['SIGNALCHAIN'][0]['PROCESSOR'][0]['CHANNEL_INFO'])[0]
					for info in nm['PROCESSOR']:
						if 'CHANNEL' in info.keys():
							if info['CHANNEL'][0]['@record'] == '1':
								chs +=1
		else:
			for info in d['SETTINGS'][1]['SIGNALCHAIN'][0]['PROCESSOR'][:385]:
				if 'CHANNEL' in info.keys():
					if info['CHANNEL'][0]['@record'] == '1':
						chs +=1
		return chs

#returns the root mean squared of the input data    
def RMS(data,start=0,window=0,despike=False):
	start = start * samplingRate# sampling rate
	if window == 0:
		window = len(data)
	else:
		window = window * samplingRate # sampling rate
	#chunk = filterTrace(data[start:start+window], 70, 6000, 25000, 3)[200:window]
	chunk = data[int(start):int(start)+int(window)] - np.mean(data[int(start):int(start)+int(window)])
	if despike:
		chunk = despike_trace(chunk,threshold=180)
	return np.sqrt(sum(chunk**2)/float(len(chunk)))

def despike_trace(trace,threshold_sd = 2.5,**kwargs):
	if 'threshold' in kwargs.keys():
		threshold = kwargs['threshold']
	else:
		threshold = np.mean(trace)+threshold_sd*np.std(trace)
		
	spike_times_a = mlab.cross_from_below(trace,threshold)
	spike_times_b = mlab.cross_from_below(trace,-1*threshold)
	for spike_time in np.concatenate((spike_times_b,spike_times_a)):
		if spike_time > 30 and spike_time < len(trace)-30:
			trace[spike_time - 20:spike_time + 20] = 0#np.random.uniform(-1*threshold,threshold,60)
	return trace

def spikeamplitudes_trace(trace,threshold_sd = 3.0,percentile = 0.9,**kwargs):
	if 'threshold' in kwargs.keys():
		threshold = kwargs['threshold']
	else:
		threshold = np.mean(trace)+threshold_sd*np.std(trace)
		
	spike_times_a = mlab.cross_from_below(trace,threshold)
	amps=[]
	for spike_time in spike_times_a:
		if spike_time > 30 and spike_time < len(trace)-30:
			amps.extend([np.max(np.abs(trace[spike_time-30:spike_time+30]))])
	if not len(amps) > 10:
		amps= [0]
	return np.sort(amps)[int(len(amps)*percentile)]# / 5.0

#returns the peak to peak range of the input data     
def p2p(data,start=0,window=0):
    start = start * samplingRate# sampling rate
    if window == 0:
        window = len(data)
    else:
        window = window * samplingRate # sampling rate
    chunk = data[start:start+window]
    return np.max(chunk)-np.min(chunk)
    
#computes a power spectrum of the input data
#optionally, plot the computed spectrum
def b(data,start=0,window=0,plot=False,ymin=1e-24,ymax=1e8,title='',samplingRate=2500):
    start = start * samplingRate# sampling rate
    if window == 0:
        window = len(data)
    else:
        window = window * samplingRate # sampling rate
    chunk = data[start:start+window]/1e6
    ps = np.abs(np.fft.fft(chunk))**2
    time_step = 1. / samplingRate
    freqs = np.fft.fftfreq(chunk.size, time_step)
    idx = np.argsort(freqs)
    ps = scipy.signal.savgol_filter(ps,5,3)
    if plot:
        plt.plot(freqs[idx], ps[idx]);
        plt.xlim(xmin=0.01);
        plt.ylim(ymin=ymin,ymax=ymax)
        plt.xscale('log')
        plt.yscale('log')
        plt.ylabel(r'$power\/density\/\frac{V^2}{Hz}$',color='k',fontsize=18)
        plt.xlabel(r'$frequency,\/ Hz$',color='k',fontsize=24)
        plt.tick_params(axis='both', which='major', labelsize=24)#;plt.locator_params(axis='y',nbins=6)
        plt.title(title)
    return (freqs[idx], ps[idx])

def periodogram(data,start=0,window=0,plot=False,ymin=1e-24,ymax=1e8,title='',samplingRate=2500):
    start = start * samplingRate# sampling rate
    if window == 0:
        window = len(data)
    else:
        window = window * samplingRate # sampling rate
    chunk = data[start:start+window]
    f,pXX = scipy.signal.periodogram(chunk,samplingRate,nfft=samplingRate)
    pXX = scipy.signal.savgol_filter(pXX,3,1)
    if plot:
        plt.plot(f, pXX);
        plt.xlim(xmin=0.5);
        plt.ylim(ymin=ymin,ymax=ymax)
        plt.xscale('log')
        plt.yscale('log')
        plt.ylabel(r'$power\/density\/\frac{V^2}{Hz}$',color='k',fontsize=18)
        plt.xlabel(r'$frequency,\/ Hz$',color='k',fontsize=24)
        plt.tick_params(axis='both', which='major', labelsize=24)#;plt.locator_params(axis='y',nbins=6)
        plt.title(title)
    return (f, pXX)

def welch_power(data,samplingRate=2500,start=0,window=0,plot=False,ymin=1e-24,ymax=1e8,title=''):
    start = start * samplingRate# sampling rate
    if window == 0:
        window = len(data);print(window)
    else:
        window = window * samplingRate # sampling rate
    chunk = data[start:start+window]
    f,pXX = scipy.signal.welch(chunk,samplingRate,nfft=samplingRate/2)
    #pXX = scipy.signal.savgol_filter(pXX,3,1)
    if plot:
        plt.plot(f, pXX);
        plt.xlim(xmin=0.01);
        plt.ylim(ymin=ymin,ymax=ymax)
        plt.xscale('log')
        plt.yscale('log')
        plt.ylabel(r'$power\/density\/\frac{V^2}{Hz}$',color='k',fontsize=18)
        plt.xlabel(r'$frequency,\/ Hz$',color='k',fontsize=24)
        plt.tick_params(axis='both', which='major', labelsize=24)#;plt.locator_params(axis='y',nbins=6)
        plt.title(title)
    return (f, pXX)

#measure the cross-spectral coherence between two traces. 
def coherence(x,y,samplingRate = 30000,returnval=None):
    spectrum, frequencies = mlab.cohere(x,y,Fs=float(samplingRate),NFFT=int(samplingRate)/5)
    if returnval:
        if type(returnval) is float:
            return np.interp(returnval,frequencies,spectrum)
        if type(returnval) is tuple:
            return np.trapz(spectrum[np.where(frequencies==returnval[0])[0]:np.where(frequencies==returnval[1])[0]],dx=5.0)  
    else:
        return (spectrum, frequencies)

def get_surface_channel_spikeband(path,start=2.,end=10.,sampling_rate=30000,plot=False,filter_size=2,sigma=1.,filter=False,probemap=None):
	mm = np.memmap(path, dtype=np.int16, mode='r')
	print(os.path.dirname(path))
	num_channels = get_channel_count(os.path.dirname(path),from_channel_map=False)
	print(num_channels)
	chunk = get_chunk(mm,start,end,num_channels,sampling_rate)
		
	if probemap is not None:
		chunk = chunk[probemap,:]
		plt.imshow(chunk[:,:30000]);plt.gca().set_aspect(100)
		plt.figure()
		
	rms = []
	good_channels = []
	for ch in range(np.shape(chunk)[0]):
		if ch not in skip_channels:
			if filter:
				data = filtr(chunk[ch,:],300,6000,sampling_rate,3)
			else:
				data = chunk[ch,:]
			rms.extend([RMS(data)])
			good_channels.extend([ch])
			
	threshold = np.mean(gaussian_filter1d(rms,filter_size)[::-1][:5])+np.std(gaussian_filter1d(rms,filter_size)[::-1][:5])*sigma		#assumes the last 5 are out of the brain; uses the mean + sd of these 5 as the threshold for pial surface
	
	# print(np.where(np.array(rms)<8.))
	# print(good_channels[np.where(np.array(rms)<8.)[0].astype(int)])
	if plot:
		plt.plot(good_channels,gaussian_filter1d(rms,filter_size))
		plt.gca().axhline(threshold,color='r')
		plt.xlabel('channel number')
		plt.ylabel('spike band RMS')
		#print(np.where(np.array(rms)<6.))
	del mm
	try:
		surface_channel = good_channels[mlab.cross_from_above(gaussian_filter1d(rms,filter_size),threshold)[0]]
		return surface_channel
	except:
		return None

def get_surface_channel_gamma(path,start=2.,end=10.,sampling_rate=2500,plot=False):
	mm = np.memmap(path, dtype=np.int16, mode='r')
	num_channels = get_channel_count(os.path.dirname(path))
	chunk = get_chunk(mm,start,end,num_channels,sampling_rate)
	
	gm = []
	good_channels = []
	for ch in range(np.shape(chunk)[0]):
		if ch not in skip_channels:
			f,pXX = welch_power(chunk[ch,:],start=2,window=8)
			gm.extend([pXX[np.where(f>40.)[0][0]]])
			good_channels.extend([ch])
	threshold = np.max(gm[::-1][:5])	#assumes the last 5 are out of the brain; uses the max gamma on these channels as the threshold
	surface_channel = good_channels[mlab.cross_from_above(gaussian_filter1d(gm,0),threshold)[0]]

	if plot:
		plt.plot(good_channels,gaussian_filter1d(gm,2))
		plt.gca().axhline(threshold,color='r')
	del mm
	return surface_channel

def get_surface_channel_freq(path,frequency_range=[1,5],start=2.,end=10.,sampling_rate=2500,filter_size=2,sigma=2.,plot=False,filter=False,probemap=None):
	mm = np.memmap(path, dtype=np.int16, mode='r')
	num_channels =  get_channel_count(os.path.dirname(path),from_channel_map=False)
	chunk = get_chunk(mm,start,end,num_channels,sampling_rate)
	if probemap is not None:
		chunk = chunk[probemap,:]
	gm = []
	good_channels = []
	for ch in range(np.shape(chunk)[0]):
		if ch not in skip_channels:
			if filter:
				data = filtr(chunk[ch,:],0.1,300,sampling_rate,3)
			else:
				data = chunk[ch,:]
			f,pXX = welch_power(chunk[ch,:],start=2,window=8)
			gm.extend([np.mean(pXX[np.where((f>frequency_range[0])&(f<frequency_range[1]))[0]])])
			good_channels.extend([ch])
	#threshold = np.mean(gm[::-1][:5])	#assumes the last 5 are out of the brain; uses the max gamma on these channels as the threshold
	threshold = np.mean(gaussian_filter1d(gm,filter_size)[::-1][:5])+np.std(gaussian_filter1d(gm,filter_size)[::-1][:5])*sigma

	if plot:
		plt.plot(good_channels,gaussian_filter1d(gm,filter_size))
		plt.gca().axhline(threshold,color='r')
		plt.xlabel('channel number')
		plt.ylabel('power in '+str(frequency_range[0])+' to '+str(frequency_range[1])+' band')
	try:
		surface_channel = good_channels[mlab.cross_from_above(gaussian_filter1d(gm,filter_size),threshold)[-1]]
		return surface_channel
	except:
		return None
	del mm
	return surface_channel

def get_probe_freq(path,frequency_range=[1,5],start=2.,end=10.,sampling_rate=2500.,filter=False,probemap=None):
	mm = np.memmap(path, dtype=np.int16, mode='r')
	num_channels = get_channel_count(os.path.dirname(path),from_channel_map=False)
	chunk = get_chunk(mm,start,end,num_channels,sampling_rate)
	if probemap is not None:
		chunk = chunk[probemap,:]
	gm = []
	good_channels = []
	for ch in range(np.shape(chunk)[0]):
		if ch not in skip_channels:
			if filter != False:
				data = filtr(chunk[ch,:],filter[0],filter[1],sampling_rate,3)
			else:
				data = chunk[ch,:]
			f,pXX = welch_power(chunk[ch,:],start=2,window=8)
			gm.extend([np.mean(pXX[np.where((f>frequency_range[0])&(f<frequency_range[1]))[0]])])
			good_channels.extend([ch])

	del mm
	return gm

def get_probe_spikeband(path,start=2.,end=10.,sampling_rate=30000,plot=False,filter_size=2,sigma=1.,filter=False,probemap=None):
	mm = np.memmap(path, dtype=np.int16, mode='r')
	num_channels = get_channel_count(os.path.dirname(path),from_channel_map=False)
	#print num_channels
	chunk = get_chunk(mm,start,end,num_channels,sampling_rate)
		
	if probemap is not None:
		chunk = chunk[probemap,:]
		plt.imshow(chunk[:,:30000]);plt.gca().set_aspect(100)
		plt.figure()
		
	rms = []
	good_channels = []
	for ch in range(np.shape(chunk)[0]):
		if ch not in skip_channels:
			if filter:
				data = filtr(chunk[ch,:],300,6000,sampling_rate,3)
			else:
				data = chunk[ch,:]
			rms.extend([RMS(data)])
			good_channels.extend([ch])
	del mm
	return rms
#=================================================================================================

#=================================================================================================
#=======plotting=============================================================================
#=================================================================================================
def get_chunk(mm,start,end,channels,sampling_rate=30000):
	chunk = mm[int(start*sampling_rate*int(channels)):int(np.floor(end*sampling_rate*(int(channels))))]
	#print np.shape(chunk)
	return np.reshape(chunk,(int(channels),-1),order='F')  * 0.195

def get_duration(mm,channels,sampling_rate=30000.):
    chunk=np.reshape(mm,(int(channels),-1),order='F')
    return chunk.shape[1]/float(sampling_rate)

def get_spike(spks_path,times,number_channels,pre=0.015,post=0.025,sampling_rate=30000):
    mm = np.memmap(spks_path, dtype=np.int16, mode='r')
    average = np.zeros((number_channels,int(sampling_rate*(pre+post))))
    count=0
    for time in times:
        try:
            temp = traces.get_chunk(mm,time-pre,time+post,number_channels,sampling_rate=sampling_rate)
            average += temp
            count+=1
        except:
            pass
    temp = temp/float(count)
    spikes_average = np.array(average.T - np.mean(average,1).T).T
    del(mm)

def get_lfp(lfp_path,times,number_channels,pre=0.015,post=0.025,sampling_rate=2500):
    mm = np.memmap(lfp_path, dtype=np.int16, mode='r')
    average = np.zeros((number_channels,int(sampling_rate*(pre+post))))
    count=0
    for time in sub_times:
        try:
            temp = traces.get_chunk(mm,time-pre,time+post,number_channels,sampling_rate=sampling_rate)
            average += temp
            count+=1
        except:
            pass
    temp = temp/float(count)
    lfp_average = np.array(average.T - np.mean(average,1).T).T
    del(mm)

def make_range_slider(data,start,window,num_channels=384,channels = [10],sampling_rate=2500,y_spacing=500,CAR=False):
    fig,ax=plt.subplots(figsize=(20,5))
    chunk = get_chunk(data,start,start+window,num_channels,sampling_rate=2500)
    if CAR:
        chunk_CAR = np.mean(chunk, axis=0)
    x = np.linspace(int(start),(start+window),int(window*sampling_rate))
    for i,ch in enumerate(channels):
        chunkch = chunk[ch,:]
        offset = np.mean(chunkch)
        if CAR: chunkch = chunkch - chunk_CAR
        ax.plot(x,chunkch-offset+i*y_spacing,'k',lw=.5)
#=================================================================================================



# -*- coding: utf-8 -*-
"""
Created on Fri Oct 31 10:39:18 2014

@author: danieljdenman
"""
import h5py
import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage as ndimage
import cv2
import os

def loadEyeTraces(filepath):
    eyedata={}
    f = h5py.File(filepath,'r')
    for i,key in enumerate(f.keys()):
        if key in ['led_positions','pupil_positions','pupil_area']:
            eyedata[key] = np.array(f[key]).astype(np.int16)
    f.close()
    #f = filepath.replace('h5')
    #timestamps from text files
    return eyedata
    
def plotEyePosition(eyedata,mn=-80,mx=10):
    plt.figure()
    bins = 20
    x = eyedata['pupil_positions'][:,0] - eyedata['led_positions'][:,0]
    y = eyedata['pupil_positions'][:,1] - eyedata['led_positions'][:,1]
    plt.plot(x,y,'k-')
    plt.xlim(mn,mx);plt.ylim(mn,mx)
    plt.figure()
    plt.plot(np.linspace(x.min(),x.max(),num=bins),np.histogram(x,bins)[0])
    plt.figure()
    plt.plot(np.linspace(y.min(),y.max(),num=bins),np.histogram(y,bins)[0])
    plt.figure()
    plt.imshow(np.histogram2d(x,y,bins)[0],cmap=plt.cm.hot,interpolation='none')
    
def smoothLED(eyedata):
    eyedata['led_positions'][:,1] = ndimage.filters.gaussian_filter1d(eyedata['led_positions'][:,1],30)
    eyedata['led_positions'][:,0] = ndimage.filters.gaussian_filter1d(eyedata['led_positions'][:,0],30)
    
def smoothPupil(eyedata):
    eyedata['pupil_positions'][:,1] = ndimage.filters.gaussian_filter1d(eyedata['pupil_positions'][:,1],10)
    eyedata['pupil_positions'][:,0] = ndimage.filters.gaussian_filter1d(eyedata['pupil_positions'][:,0],10)
    
def trigger(eyedata,triggers,window,**kwargs):
    eyeOut={}
    eyeOut['pupil_positions']=[]
    eyeOut['led_positions']=[]
    eyeOut['pupil_area']=[]
    if 'combine' in kwargs.keys():
        f = plt.gcf()
        plt.figure(f.number)
    for trig in triggers:
        
        eyeOut['pupil_positions'].append(eyedata['pupil_positions'][trig:trig+window,:])
        eyeOut['led_positions'].append(eyedata['led_positions'][trig:trig+window,:])
        eyeOut['pupil_area'].append(eyedata['pupil_area'][trig:trig+window])
        
        #print trig
        if 'plot' in kwargs.keys() and kwargs['plot']=='position':
            if 'color' in kwargs.keys():
                color = ''.join((kwargs['color'],'-o'))
            else:
                color =''.join(('k','-o'))
            traceX = np.array(eyedata['pupil_positions'][trig:trig+window,0]-eyedata['led_positions'][trig:trig+window,0])
            traceY = np.array(eyedata['pupil_positions'][trig:trig+window,1]-eyedata['led_positions'][trig:trig+window,1])            
            plt.plot(traceX-traceX[0],
                     traceY-traceY[0],
                        color)
            plt.ylim(-30,30);plt.xlim(-30,30)
        
        if 'plot' in kwargs.keys() and kwargs['plot']=='area':
            if 'color' in kwargs.keys():
                color = kwargs['color'] 
            else:
                color = 'k'
            plt.plot(range(window),eyedata['pupil_area'][trigger:trigger+window],''.join((color,'-o')));plt.ylim(-100,-10);plt.xlim(-100,-10)
    
    return eyeOut
#movie:    
def triggerMovie(movie,triggers,numberofframes,name='output',**kwargs):          
    
    allframes = [np.linspace(start,start+numberofframes-1,numberofframes) for start in triggers] 
    framestowrite = np.concatenate(allframes).astype(int)
    currentframe = 0
    
    # Define the codec and create the VideoCapture and VideoWriter objects
    cam=cv2.VideoCapture(movie) 
    size = (int(cam.get(cv2.cv.CV_CAP_PROP_FRAME_WIDTH)),
           int(cam.get(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT)))
    print size
    fourcc = cv2.cv.CV_FOURCC(*'DIVX')
    out = cv2.VideoWriter(os.path.join(os.path.dirname(movie),name+'.avi'),fourcc, 15.0, size)
    
    #read, display and writing only the frames defined by triggers and number of frames 
    while True:
        ret, img = cam.read()                      
        if (type(img) == type(None)):
            break

        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        
        if currentframe in framestowrite:
            out.write(img)
            cv2.imshow('frame', gray)
        
        #print ret
        currentframe+=1
        if (0xFF == ord('q') & cv2.waitKey(1)) or img.size == 0:
            break
    cam.release()
    out.release()   
    cv2.destroyAllWindows()
    
def triggerMovieAvg(movie,triggers,numberofframes,name='output',**kwargs):          
    
    allframes = [np.linspace(start,start+numberofframes-1,numberofframes) for start in triggers] 
    framestowrite = np.concatenate(allframes).astype(int)
    currentframe = 0

    
    # Define the codec and create the VideoCapture and VideoWriter objects
    cam=cv2.VideoCapture(movie) 
    size = (int(cam.get(cv2.cv.CV_CAP_PROP_FRAME_WIDTH)),
           int(cam.get(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT)))
    print size
    fourcc = cv2.cv.CV_FOURCC(*'DIVX')
    out = cv2.VideoWriter(os.path.join(os.path.dirname(movie),name+'_trialaverage.avi'),fourcc, 15.0, size)
    
    avg = [np.zeros((size[0],size[1],3)) for i in range(numberofframes)]
    frameno = 0
    #read, display only the frames defined by triggers and number of frames 
    while True:
        ret, img = cam.read()                      
        if (type(img) == type(None)):
            break
        
        if currentframe in framestowrite :
            #out.write(img)
            cv2.imshow('frame', img)
            avg[frameno]+=img
            frameno+=1
        else:
            frameno = 0
            
        #print ret
        currentframe+=1
        if (0xFF == ord('q') & cv2.waitKey(1)) or img.size == 0:
            break
        
    cam.release()
    
    averagemovie = [img / len(triggers) for img in avg] # average
    
    #writing only the frames defined by triggers and number of frames 
    for img in averagemovie:
        out.write(img.astype('u1'))
    out.release()   
    
    cv2.destroyAllWindows()
        
#    startFrame = triggers[0]
#    for frame in numFramesInMovie:
#        if frame in np.linspace(startFrame,startFrame+numberofframes-1,numberofframes).astype(int):
#            #start writing 
#
#            start           
            
#plt.plot(o0['mean'][0]-o0['mean'][0][0],o0['mean'][1]-o0['mean'][1][0])
#plt.plot(o45['mean'][0]-o45['mean'][0][0],o45['mean'][1]-o45['mean'][1][0])
#plt.plot(o90['mean'][0]-o90['mean'][0][0],o90['mean'][1]-o90['mean'][1][0])
#plt.plot(o135['mean'][0]-o135['mean'][0][0],o135['mean'][1]-o135['mean'][1][0])
    



from pynwb import NWBHDF5IO
import h5py, glob, os
import numpy as np
from ellipse import LsqEllipse
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import seaborn as sns
import tqdm
from p_tqdm import p_map
import time
import pandas as pd
from pynwb import TimeSeries
from pynwb.behavior import PupilTracking
from hdmf.backends.hdf5.h5_utils import H5DataIO
from datetime import datetime
from dateutil.tz import tzlocal
from pynwb import NWBFile, TimeSeries, NWBHDF5IO, ProcessingModule
from pynwb.behavior import PupilTracking
from joblib import Parallel, delayed

def get_eye_frame_from_NWB(nwb,index):
    positions = ['DR','D','LD','L','RV','R','VL','V']
    x,y = [],[]
    for p in positions:
        pos = np.array(nwb.processing.get('behavior').get('PupilTracking')[p+'pupil_positions'].data)
        x.extend([pos[index,0]])
        y.extend([pos[index,1]])
    return x,y
def get_eye_frame(df,index):
    positions = ['DR','D','LD','L','RV','R','VL','V']
    x,y = [],[]
    for p in positions:
        x_ = df[p+'pupil_X'].values[index]
        y_ = df[p+'pupil_Y'].values[index]
        x.extend([x_])
        y.extend([y_])
    return x,y
def fit_ellipse(x,y):
    lsqe = LsqEllipse()
    reg = lsqe.fit( np.array(list(zip(x, y))))
    center, width, height, phi = reg.as_parameters()
    return center, width, height, phi

    
def check_OKR_NWB(recording_folder,
                 experimenter = 'j_h',
                 experiment_description= 'Denman Lab (Hughes Lab collaboration) on Autobahn Therepeutics remyelination therapy.'):
    if len(glob.glob(os.path.join(recording_folder,'*resnet*'))) > 2: 
        # print('making NWB for '+recording_folder)

            
        nwb_path = os.path.join('/Volumes/s1/autobahn/nwbs/okr/with_center',os.path.basename(recording_folder))+'_c.nwb'
        if os.path.exists(nwb_path): print('FOUND: '+recording_folder)
        else: print('MISSING: '+recording_folder)
    return None

#walk folder tree
rootDir = '/Volumes/s1/OKR/sessions'

for dirName, subdirList, fileList in os.walk(rootDir):
    if len(glob.glob(os.path.join(dirName,'*resnet*'))) > 2:
        if 'Cohort' in dirName and 'Prelim' not in dirName:# and '0' not in dirName:
            if len(glob.glob(os.path.join(dirName,'*resnet*'))) > 2:
                # print('Found directory: %s' % dirName)
                check_OKR_NWB(dirName) #do the work
            else: pass#print('NO DLC data for '+dirName)

# Parallel(n_jobs=7)(make_OKR_NWB(dirName) for dirName, subdirList, fileList in os.walk(rootDir)) #do the work)
# make_OKR_NWB('/Volumes/s1/OKR/sessions/Cohort1/Week1Data/3582(wk1.1of2)/3582(wk1.1of2)_2021_8_17-19_24_41')


import numpy as np
import matplotlib.pyplot as plt
import h5py, os, sys, glob, tqdm, datetime
import _pickle as pkl
import pandas as pd

import pynwb
from hdmf.backends.hdf5.h5_utils import H5DataIO
from hdmf.utils import StrDataset
from pynwb import NWBFile, NWBHDF5IO, TimeSeries, ProcessingModule
from pynwb.device import Device
from pynwb.epoch import TimeIntervals
from pynwb.ecephys import ElectrodeGroup
from pynwb.behavior import BehavioralEvents, BehavioralEpochs, BehavioralTimeSeries, PupilTracking, IntervalSeries


from dateutil.tz import tzlocal
from pynwb import NWBFile


#class data object made by dan for our task. 
#input when making a new instance is a path (string) to the data in the Deepnote file tree
#e.g., s = session('/data/thisisafolder/thisisthefolderthecsvfilesarein')
class Session:
    def __init__(self,path):
        self.path = path.lower()
        self.df_pulls = pd.read_csv(os.path.join(path,'lever_pulls.csv'))
        self.df_pushes = pd.read_csv(os.path.join(path,'lever_pushes.csv'))
        self.df_releases = pd.read_csv(os.path.join(path,'lever_releases.csv'))
        self.df_rewards = pd.read_csv(os.path.join(path,'rewards.csv'),index_col=False)
        self.df_target_images = pd.read_csv(os.path.join(path,'target_images.csv'))
        self.df_trial_ends = pd.read_csv(os.path.join(path,'trial_ends.csv'))
        self.df_trial_starts = pd.read_csv(os.path.join(path,'trial_starts.csv'))

        self.df_pulls['lever_direction'] =  ['pull' for i in range(self.df_pulls.shape[0])]
        self.df_pushes['lever_direction'] =  ['push' for i in range(self.df_pushes.shape[0])]
        self.df_lever = pd.concat((self.df_pushes,self.df_pulls),ignore_index=True).sort_values('Frame')

    def clean_duplicates(self, df):
        return df.drop(np.argwhere(np.diff(df.Frame.values) < 5)[0][0]+1)

    def check_number_of_trials(self):
        if self.df_trial_starts.shape[0] == self.df_lever.shape[0] == self.df_releases.shape[0] == (self.df_target_images.shape[0]+1):
            self.num_trials = self.df_trial_starts.shape[0]
            print('number of trials: '+str(self.num_trials))
            return True
        else:
            # print('number of trial starts: '+str(self.df_trial_starts.shape[0]))
            # print('number of lever moves: '+str(self.df_lever.shape[0]))
            # print('number of releases: '+str(self.df_releases.shape[0]))
            # print('number of target images: '+str((self.df_target_images.shape[0]+1)))
            return False
    
    def make_df(self, phase, offset=0):
        if phase == 1:
            if not self.check_number_of_trials():
                
                if self.df_trial_starts.shape[0] == self.df_lever.shape[0]:
                    #make sure releases match by removing releases that happen within 3 frames of each other
                    self.df_releases = self.df_releases.iloc[np.where(np.diff(self.df_releases.Frame.values)>3)[0]]
                else: 
                    print(path+' trial starts and lever movements do not match, which is a problem for Phase1')
                    return

            #enforce target image shape
            self.df_target_images=self.df_target_images.iloc[np.where(self.df_target_images.Frame.values < np.max(self.df_trial_starts.Frame.values))[0]]
            
            #combine times for each trial
            self.df = pd.concat([self.df_trial_starts.drop('Value',axis=1).reset_index(),
                                self.df_releases.drop('Value',axis=1).reset_index(),
                            self.df_lever.drop('Value',axis=1).reset_index()],
                            axis=1,ignore_index=False)

            #rename columns
            self.df.columns = ['trial_start_index','trial_start_frame','trial_start_timestamp',
                        'release_index','release_frame', 'release_timestamp',
                        'lever_move_index','lever_move_frame','lever_move_timestamp','lever_move_direction']

            #put in target and possible reward information and times
            # reward_time = np.zeros(self.df.shape[0]);reward_time[:]=np.nan
            # for reward_frame in self.df_rewards.Frame.values:
            #     for trial in range(self.df.shape[0]):
            #         if abs(reward_frame - self.df['release_frame'][trial]) < 4:
            #             reward_time[trial] = self.df_rewards.Timestamp[self.df_rewards.Frame==reward_frame]
            # self.df['reward_timestamp']=reward_time
            self.df['target_image_int'] = [1]+np.array(self.df_target_images.Value).tolist()
            directions = ['','pull','push']
            image = ['cheetah','elephant']
            self.df['target_image'] = [image[i] for i in self.df['target_image_int']]
            self.df['target_direction'] = [directions[i] for i in self.df['target_image_int']]
        
            reward_,reward_times,reward_frames=[],[],[]
            for index, row in self.df.iterrows():
                start_frame =  self.df.trial_start_frame[index]
                try: end_frame = self.df.trial_start_frame[index+1]
                except:end_frame = self.df.trial_start_frame[index]+1000
                _ = gather_events_by_trial(self.df_rewards,start_frame,end_frame)
                reward_.extend([_[0]])
                reward_frames.extend([_[1]])
                reward_times.extend([_[2]])
            self.df['reward'] = reward_
            self.df['reward_times'] = reward_times
            self.df['reward_frames'] = reward_frames

        if phase == 2:
            self.df = self.df_trial_starts.drop('Value',axis=1)
            self.df.columns=['trial_start_time','trial_start_Timestamp']
            first_trial = self.df.trial_start_time.iloc[0]
            ind = np.where(self.df_trial_ends.Timestamp.values > first_trial)[0][0]
            trials_no = self.df.shape[0]
            print(trials_no)

            if self.df.trial_start_time.values[0] > self.df_trial_ends.Timestamp.values[0]:
                #find where the first df_trial_ends row is greater than the first trial
                self.df_trial_ends[self.df_trial_ends.Timestamp > first_trial]

            if trials_no > self.df_trial_ends.Frame.values[ind:].shape[0]:
                print('more starts than ends')
                self.end_frames = self.df_trial_ends.Frame.values[ind:].tolist()
                self.end_times = self.df_trial_ends.Timestamp.values[ind:].tolist()
                self.end_Timestamps = self.df_trial_ends.Value.values[ind:].tolist()
                print(np.shape(self.end_frames))
                for j in range(trials_no - np.shape(self.end_frames)[0]):
                    self.end_frames.extend([np.nan])
                    self.end_times.extend([np.nan])
                    self.end_Timestamps.extend([np.nan])
                print(np.shape(self.end_frames))
                self.df['trial_end_frame'] = self.end_frames
                self.df['trial_end_time'] = self.end_times
                self.df['trial_end_Timestamp'] =  self.end_Timestamps
            else:
                self.df['trial_end_frame'] = self.df_trial_ends.Frame.values[ind:]
                self.df['trial_end_time'] = self.df_trial_ends.Timestamp.values[ind:]
                self.df['trial_end_Timestamp'] = self.df_trial_ends.Value.values[ind:]
            image = ['cheetah','elephant']
            self.df['target_image'] = [image[i] for i in self.df_target_images.Value.values]

            pull_,push_,reward_ = [],[],[]
            pull_times,push_times,reward_times= [],[],[]
            push_frames,pull_frames,reward_frames= [],[],[]
            for index, row in self.df.iterrows():
                start_frame = index
                # end_frame = self.df.trial_end_frame[index+offset]
                try:
                    end_frame = self.df_trial_ends[self.df_trial_ends.Frame > start_frame].Frame.values[0]+2
                    #gather pulls
                    _ = gather_events_by_trial(self.df_pulls,start_frame,end_frame)
                    pull_.extend([_[0]])
                    pull_frames.extend([_[1]])
                    pull_times.extend([_[2]])
                    _ = gather_events_by_trial(self.df_pushes,start_frame,end_frame)
                    push_.extend([_[0]])
                    push_frames.extend([_[1]])
                    push_times.extend([_[2]])
                    _ = gather_events_by_trial(self.df_rewards,start_frame,end_frame)
                    reward_.extend([_[0]])
                    reward_frames.extend([_[1]])
                    reward_times.extend([_[2]])
                except:
                    _=[np.nan,np.nan,np.nan]
                    pull_.extend([_[0]])
                    pull_frames.extend([_[1]])
                    pull_times.extend([_[2]])
                    push_.extend([_[0]])
                    push_frames.extend([_[1]])
                    push_times.extend([_[2]])
                    _=[False,[],[]]
                    reward_.extend([_[0]])
                    reward_frames.extend([_[1]])
                    reward_times.extend([_[2]])
            self.df['pulled'] = pull_
            self.df['pushed'] = push_
            self.df['rewarded'] = reward_
            self.df['pull_times'] = pull_times
            self.df['push_times'] = push_times
            self.df['reward_times'] = reward_times
            self.df['pull_frames'] = pull_frames
            self.df['push_frames'] = push_frames
            self.df['reward_frames'] = reward_frames
            lever_move_time = []
            for trial in self.df.iterrows():
                if np.isnan(trial[1].pulled) and np.isnan(trial[1].pushed):
                    lever_move_time.extend([np.nan])
                else:
                    if trial[1].pulled:
                        if type(trial[1].pull_times) == list:
                            lever_move_time.extend([trial[1].pull_times[0]])
                        else: lever_move_time.extend([trial[1].pull_times[0]])
                    else:
                        lever_move_time.extend([trial[1].push_times[0]])
            self.df['lever_move_time']=lever_move_time

            self.df['reaction_time'] = self.df.lever_move_time - self.df.trial_start_time

        if phase > 0:
            print(self.path)
            self.mouse = self.path.split('/')[-2].lower()
            self.date = self.path.split('/')[-1]
            # dt_string= self.date.strip(self.mouse)[1:]
            dt_string = self.date[4:]
            print(self.date)
            format = "%Y_%m_%d-%H_%M_%S"
            dt_object = datetime.datetime.strptime(dt_string, format)
            self.df['phase'] = [phase]*self.df.shape[0]
            self.df['datetime'] = [dt_object]*self.df.shape[0]
            self.df['year'] = [dt_object.year]*self.df.shape[0]
            self.df['month'] = [dt_object.month]*self.df.shape[0]
            self.df['day'] = [dt_object.day]*self.df.shape[0]
            self.start_in_seconds = dt_object.hour*60*60 + dt_object.minute*60 + dt_object.second 
            self.df['start_in_seconds'] = [self.start_in_seconds]*self.df.shape[0]
            try:
                for col in self.df.columns:
                    if 'time' in col:
                        if 'trial' in col:
                            self.df[col] =  np.array(self.df[col].values) + self.tart_in_seconds
                        else:
                            self.df[col] = [np.array(np.array(ts)+self.start_in_seconds).tolist() for ts in self.df[col].values]
                        # self.df[col].values+start_in_seconds
            except: pass
            try: self.df['target_image'] = [image[i] for i in self.df['target_image_int']]
            except: pass
            self.df['mouse']=[self.mouse]*self.df.shape[0]
        
def gather_events_by_trial(df,start_frame,end_frame):
#this function takes a DataFrame as input, and searches this DataFrame 
#for any events in the Frame column that falls between the input start_frame and end_frame 
    moved, frames, times = False, [], []
    for index, row_ in df.iterrows():
        if row_.Frame > start_frame and row_.Frame < end_frame:
            moved = True
            frames.extend([row_.Frame])
            times.extend([row_.Timestamp])
    return moved, frames, times

def check_session_phase(s):
    if s.df_trial_starts.shape[0] > 0: #make sure there are some trials 
        if s.df_trial_starts.shape[0] == s.df_lever.shape[0]:# in phase 1, trials are started by lever moves, so this is equal
            #could also be 2c, which is 2AFC! need to check
            return 1
        else:
            #TODO: differentiate between 2a and 2b and 3
            return 2
    else: return 0


def combine_Sessions(sessions):
#input: sessions is a list of Session objects created by dlabbehavior cheetah_elephant Session
#outpu
    sessions[0].make_df(2)
    df = sessions[0].df
    for s in sessions[1:]:
        s.make_df(2)
        df_toadd = s.df
        end_of_last_session_t = df.trial_end_time.values[-1]
        end_of_last_session_f = df.trial_end_frame.values[-1]
        for col in ['trial_start_time','trial_end_time','lever_move_time']:
            df_toadd[col] = df_toadd[col] + end_of_last_session_t
        
        for col in ['pull_times','push_times','reward_times']:
            df_toadd[col] = [np.array(np.array(times) + end_of_last_session_t).tolist for times in df_toadd[col]]
        
        df_toadd.trial_end_frame = df_toadd.trial_end_frame.values + end_of_last_session_f
        
        for col in ['pull_frames','push_frames','reward_frames']:
            df_toadd[col] = [np.array(np.array(frames) + end_of_last_session_f).tolist for frames in df_toadd[col]]
        
        df = pd.concat([df,df_toadd],ignore_index=True)
    return df

def sort_day_sessions(paths):
    new_names = []
    for path in paths:
        session_time_string = os.path.basename(path).split('-')[-1]
        if len(session_time_string.split('_')[0]) < 2:
            hour = '0'+ session_time_string.split('_')[0]
        else: hour = session_time_string.split('_')[0]
        if len(session_time_string.split('_')[1]) < 2:
            minute = '0'+ session_time_string.split('_')[1]
        else: minute = session_time_string.split('_')[1]
        if len(session_time_string.split('_')[2]) < 2:
            second = '0'+ session_time_string.split('_')[2]
        else: second = session_time_string.split('_')[2]
#os.path.basename(path).split('-')[0]+'-'+
        new_names.extend([hour+'_'+minute+'_'+second])
    return np.array(paths)[np.argsort(new_names).astype(int)]


def create_nwb(recording_folder):
    s = Session(recording_folder)

    s.make_df(2)#<--- change t

    nwb_file = NWBFile('Denman Lab plaid discrimination', 
                    recording_folder, 
                    datetime.datetime.now(tzlocal()),
                    experimenter=experimenter,
                    lab='Denman Lab',
                    institution='University of Colorado',
                    experiment_description=experiment_description,
                    session_id=os.path.basename(recording_folder))

    behavior_module = ProcessingModule('behavior', 'behavior module')
    nwb_file.add_processing_module(behavior_module)

    nwb_file.add_trial_column('image','image presented on this trial.')
    nwb_file.add_trial_column('imageApercent','contrast of one grating presented on this trial.')
    nwb_file.add_trial_column('imageBpercent','contrast of one grating presented on this trial.')
    nwb_file.add_trial_column('pulled','time in seconds that the trial ended.')
    nwb_file.add_trial_column('pushed','time in seconds that the trial ended.')
    nwb_file.add_trial_column('response_time','time in seconds that the trial ended.')
    nwb_file.add_trial_column('rewarded','if a reward was delivered on this trial')

    for i,unit_row in s.df.iterrows():
        if unit_row.target_image == 'elephant':
            percA= 0; percB= 100
        else: 
            percA= 100; percB= 0
        nwb_file.add_trial(start_time=unit_row.trial_start_time,
                        stop_time=unit_row.trial_end_time,
                        image='grating',
                        imageApercent=str(percA),
                        imageBpercent=str(percB),
                        pulled=str(unit_row.pulled),
                        pushed=str(unit_row.pushed),
                        response_time=str(unit_row.lever_move_time),
                        rewared=unit_row.rewarded)
    try: 
        lf = pd.read_csv(os.path.join(recording_folder,'buffered_lever_frames.csv'),index_col=False)
        val_ = [int(vala[1:])for vala in lf.Value]
        lf.Value = val_
        
        lever_ts = TimeSeries(
            name='lever_position',
            timestamps=lf.Timestamp.values,
            data=lf.Value.values,
            unit='arbitrary',
            description='The position reading of the 4 way encoder attached to the lever the mouse pushes forward, backward, left and right with his forelimbs.',
            comments='Lever movement. 512 is centered.'
        )
        nwb_file.add_acquisition(lever_ts)
    except:
        print('failed to add lever data to '+recording_folder)

    nwb_path = os.path.join(recording_folder,os.path.basename(recording_folder))+'_plaid.nwb'
    with pynwb.NWBHDF5IO(nwb_path, 'w') as io:
        io.write(nwb_file)


if __main__:
    recording_folder = '/Users/danieljdenman/Desktop/c70/c70_2021_10_30-17_3_12'
    experimenter = 'djd'
    experiment_description= 'Denman Lab plaid direction discrimination'



from dlab.generalephys import option234_positions
import numpy as np
import os
def load_phy_template(path,site_positions = option234_positions,**kwargs):
# load spike data that has been manually sorted with the phy-template GUI
# the site_positions should contain coordinates of the channels in probe space. for example, in um on the face of the probe
# returns a dictionary of 'good' units, each of which includes:
#	times: spike times, in seconds
#	template: template used for matching
#	ypos: y position on the probe, calculated from the template. requires an accurate site_positions. averages template from 100 spikes.
#	xpos: x position on the probe, calcualted from the template. requires an accurate site_positions. averages template from 100 spikes.
	clusters = np.load(open(os.path.join(path,'spike_clusters.npy')))
	spikes = np.load(open(os.path.join(path,'spike_times.npy')))
	spike_templates = np.load(open(os.path.join(path,'spike_templates.npy')))
	templates = np.load(open(os.path.join(path,'templates.npy')))
	cluster_id = [];
	[cluster_id.append(row) for row in csv.reader(open(os.path.join(path,'cluster_groups.csv')))];
	if 'sampling_rate' in kwargs.keys():
		samplingrate = kwargs['sampling_rate']
	else:
		samplingrate =30000.
		print('no sampling rate specified, using default of 30kHz')
		
	units = {}
	for i in np.arange(1,np.shape(cluster_id)[0]):
		if cluster_id[i][0].split('\t')[1] == 'good' :#:or cluster_id[i][0].split('\t')[1] == 'unsorted' :#if it is a 'good' cluster by manual sort
			unit = int(cluster_id[i][0].split('\t')[0])
			units[str(unit)] = {}
			
			#get the unit spike times
			units[str(unit)]['times'] = spikes[np.where(clusters==unit)]/samplingrate
			units[str(unit)]['times'] = units[str(unit)]['times'].flatten()
			
			#get the mean template used for this unit
			all_templates = spike_templates[np.where(clusters==unit)].flatten()
			n_templates_to_subsample = 100
			random_subsample_of_templates = templates[all_templates[np.array(np.random.rand(n_templates_to_subsample)*all_templates.shape[0]).astype(int)]]
			mean_template = np.mean(random_subsample_of_templates,axis=0)
			units[str(unit)]['template'] = mean_template
			
			#take a weighted average of the site_positions, where the weights is the absolute value of the template for that channel
			#this gets us the x and y positions of the unit on the probe.
			weights = np.zeros(site_positions.shape)
			for channel in range(site_positions.shape[0]):
				weights[channel,:]=np.trapz(np.abs(mean_template.T[channel,:]))
			weights = weights/np.max(weights)
			low_values_indices = weights < 0.25  # Where values are low,
			weights[low_values_indices] = 0      # make the weight 0
			(xpos,ypos)=np.average(site_positions,axis=0,weights=weights)
			units[str(unit)]['waveform_weights'] = weights
			units[str(unit)]['xpos'] = xpos
			units[str(unit)]['ypos'] = ypos #- site_positions[-1][1]
	return units



import numpy as np
import pandas as pd
import warnings
import glob, os, h5py, csv
from dlab.generalephys import option234_positions
from dlab.sglx_analysis import readAPMeta

try:
    from nwb.nwb import NWB
    from nwb.nwbts import TimeSeries
except:
    print('no Allen Institute NWB API. get this from http://stash.corp.alleninstitute.org/projects/INF/repos/ainwb/browse')
try:
    from djd.OpenEphys import loadContinuous, loadFolder, load_kwik_klusters
except:
    try:
        from OpenEphys import loadContinuous, loadFolder, load_kwik_klusters
    except:
        print('no OpenEphys.py get this from https://github.com/open-ephys/analysis-tools')

warnings.simplefilter(action='ignore', category=FutureWarning)

# def remap_data(data,electrodemap):
#     if type(data) - dict:
#         out = np.zeros(np.shape(data[data.keys()[10]])[0],len(electrodemap))
#         for ch in electrodemap:
#             out[:,ch] = data[np.sort(data.keys())[ch]]
#         return out

# class ExtracellularEphysFile(NWB):
#     def __init__(self,**kwargs):
#         modify=False
#         if 'meta' in kwargs.keys():
#             if 'modify' in kwargs.keys():
#                 modify=True
#             NWB.__init__(self,modify=modify,**kwargs['meta'])
#             self.meta = kwargs['meta']
#         else:
#             print('please provide information about the ephys data. use meta = {information about data, including at least "sort_method" and "sampling_rate"}')
#             return None
        
#     # THIS DOESN'T REALLY WORK for most kinds of data.
#     def add_raw_ephys(self,path,electrodemap=None,group=None,samplingrate=None,name='raw extracellualr ephys data'):
#         if samplingrate == None:
#             samplingrate = self.meta['sampling_rate']
#         if '.' in os.path.basename(path):
#             if os.path.basename(path).split('.')[-1] == 'dat':
#                 pass
            
#             if os.path.basename(path).split('.')[-1] == 'kwd':
#                 slf = self.file_pointer
#                 kwikfile = h5py.File(path,'r')
#                 if group == None:
#                     kwikfile.copy(kwikfile['recordings']['0'],slf['acquisition/timeseries'])
#                     slf.create_dataset('acquisition/timeseries/samplingrate',data=samplingrate)
#                 else:
#                     slf.create_group('acquisition/timeseries/'+group)
#                     kwikfile.copy(kwikfile['recordings']['0'],slf['acquisition/timeseries/'+group])
#                     slf.create_dataset('acquisition/timeseries/'+group+'/samplingrate',data=samplingrate)
                
#         else:     
#             data = loadFolder(path)
#             if electrodemap is not None:
#                 data = remap_data(data,electrodemap)
                
#             acquisition = self.create_timeseries('ElectricalSeries',name,'acquisition')
#             acquisition.set_data(data)
#             acquistion.close
    
#     # function for adding the clustering of some ephys data to an NWB file.
#     # takes a directory as input
#     # uses the 'sort_method' [defaults to the 'sort_method' in the metadata if no method provided.]
#     # options for 'sort method' are:
#     #       'phy':...placeholder, doesn't do anything right now...
#     #       'phy-template': expects cluster_groups.csv, spike_clusters.npy, and spike_times.npy in the directory
#     #       'kilosort': expects spike_templates.npy, and spike_times.npy in the directory. use only for the direct outputs of kilosort, that haven't been modified with phy-template
#     #       'spyking-circus': expects a .kwik and corresponding .kwx in the directory
#     #       'clu': expects spike time information (a dictionary with spiketimes for each unit) to be input with add_clustering(times=spiketime_information)
#     # places the results in the "processing" field of the NWB, under the name provided [default: 'extracellular electrophysiology data']
#     def add_clustering(self,path,**kwargs):
#         if 'sort_method' in kwargs.keys():
#             method = kwargs['sort_method']
#         else:
#             method = self.meta['sort_method']
            
#         if method == 'phy':
#             pass
#         if 'spyking-circus' in method:
#            self.spikes_data = load_kwik_klusters(path,phy_v=2)
#            self.spiketime_information = {}
#            self.all_spiketimes = []
#            self.all_cluster_numbers = []
#            for unit_name in self.spikes_data:
#               if type(self.spikes_data[unit_name]) == dict:
#                   if 'type' in self.spikes_data[unit_name].keys():
#                       if self.spikes_data[unit_name]['type'] == 'unit':
#                          self.spiketime_information[unit_name] = {}
#                          self.spiketime_information[unit_name]['spike_times'] = np.array(self.spikes_data[unit_name]['times'])/self.meta['sampling_rate']
#                          self.all_spiketimes.extend(np.array(np.array(self.spikes_data[unit_name]['times'])/self.meta['sampling_rate']).tolist())
#                          self.all_cluster_numbers.extend(np.array(np.ones(len(self.spikes_data[unit_name]['times']))*int(unit_name)).tolist())
        
#         if method == 'clu':
#            if 'times' in kwargs.keys():
#                spiketime_information = kwargs['times']
#            else:
#                raise ValueError('times not specified. use times = spiketime_information as an input to add_unit_times().')
        
#         if 'phy-template' in method:
#             self.all_spiketimes = []
#             self.all_cluster_numbers = []
            
#             self.clusters_data = np.load(open(os.path.join(path,'spike_clusters.npy')))
#             self.spikes_data = np.load(open(os.path.join(path,'spike_times.npy')))
#             spike_templates = np.load(open(os.path.join(path,'spike_templates.npy')))
#             templates = np.load(open(os.path.join(path,'templates.npy')))
            
#             cluster_groups = []
#             [cluster_groups.append(row) for row in csv.reader(open(os.path.join(path,'cluster_groups.csv')))];
#             self.spiketime_information = {}
#             for i in np.arange(1,np.shape(cluster_groups)[0]):
#                 if cluster_groups[i][0].split('\t')[1] == 'good':           #if it is a 'good' cluster by manual sort
#                     unit = int(cluster_groups[i][0].split('\t')[0])
#                     self.spiketime_information[str(unit)] = {}
                    
#                     self.spiketime_information[str(unit)]['spike_times'] = self.spikes_data[np.where(self.clusters_data==unit)]/self.meta['sampling_rate']    
#                     self.spiketime_information[str(unit)]['spike_times']= self.spiketime_information[str(unit)]['spike_times'].flatten()
                    
#                     #get the mean template used for this unit
#                     all_templates = spike_templates[np.where(self.clusters_data==unit)].flatten()
#                     n_templates_to_subsample = 100
#                     random_subsample_of_templates = templates[all_templates[np.array(np.random.rand(n_templates_to_subsample)*all_templates.shape[0]).astype(int)]]
#                     mean_template = np.mean(random_subsample_of_templates,axis=0)
#                     self.spiketime_information[str(unit)]['template'] = mean_template
                    
#                     #take a weighted average of the channelmap, where the weights is the absolute value of the template for that channel
#                     #this gets us the x and y positions of the unit on the probe.
#                     if 'channelmap' in kwargs.keys():
#                         channelmap = kwargs['channelmap']
#                         weights = np.zeros(channelmap.shape)
#                         for channel in range(channelmap.shape[0]):
#                             weights[channel,:]=np.trapz(np.abs(mean_template.T[channel,:]))
#                         weights = weights/np.max(weights)
#                         (xpos,ypos)=np.average(channelmap,axis=0,weights=weights)
#                     else:
#                         (xpos,ypos)=(np.nan,np.nan)
#                     self.spiketime_information[str(unit)]['xpos'] = xpos + 6
#                     self.spiketime_information[str(unit)]['ypos'] = ypos - channelmap[-1][1]
                    
#                     self.all_spiketimes.extend(self.spiketime_information[str(unit)]['spike_times'])
#                     self.all_cluster_numbers.extend(np.array(np.ones(len(self.spiketime_information[str(unit)]['spike_times']))*unit).tolist())



			
#         if 'kilosort' == method:
#             self.all_spiketimes = []
#             self.all_cluster_numbers = []
#             self.clusters = np.load(open(os.path.join(path,'spike_clusters.npy')))
#             self.clusters_data = np.load(open(os.path.join(path,'spike_templates.npy')))
#             self.spikes_data = np.load(open(os.path.join(path,'spike_times.npy')))
#             self.templates =  np.load(open(os.path.join(path,'templates.npy')))
#             cluster_groups = []
#             if os.path.isfile(os.path.join(path,'cluster_groups.csv')):
#                 cluster_groups = [row for row in csv.reader(open(os.path.join(path,'cluster_groups.csv')))][1:];
#             else:
#                 [cluster_groups.append(str(row)+'\tgood') for row in np.unique(self.clusters_data)] # fake that all the clusters are good.
#             self.spiketime_information = {}
#             for i in np.arange(1,np.shape(cluster_groups)[0]):
#                 if cluster_groups[i][0].split('\t')[1] == 'good':           #if it is a 'good' cluster by manual sort
#                     unit = int(cluster_groups[i][0].split('\t')[0])
#                     self.spiketime_information[str(unit)] = {}
#                     self.spiketime_information[str(unit)]['spike_times'] = self.spikes_data[np.where(self.clusters==unit)]/self.meta['sampling_rate']    
#                     self.spiketime_information[str(unit)]['spike_times']= self.spiketime_information[str(unit)]['spike_times'].flatten()
#                     self.all_spiketimes.extend(self.spiketime_information[str(unit)]['spike_times'])
#                     self.all_cluster_numbers.extend(np.array(np.ones(len(self.spiketime_information[str(unit)]['spike_times']))*unit).tolist())
                    
#                     #get the mean template used for this unit
#                     if 'site_positions' in kwargs.keys():
#                         site_positions = kwargs['site_positions']
#                         if 'offset' in kwargs.keys():
#                             offset = kwargs['offset']
#                         else:
#                             offset = 0
#                         all_templates = self.clusters_data[np.where(self.clusters==unit)].flatten()
#                         n_templates_to_subsample = 100
#                         random_subsample_of_templates = self.templates[all_templates[np.array(np.random.rand(n_templates_to_subsample)*all_templates.shape[0]).astype(int)]]
#                         mean_template = np.mean(random_subsample_of_templates,axis=0)
                        
#                         #take a weighted average of the site_positions, where the weights is the absolute value of the template for that channel
#                         #this gets us the x and y positions of the unit on the probe.
#                         weights = np.zeros(site_positions.shape)
#                         for channel in range(site_positions.shape[0]):
#                             weights[channel,:]=np.trapz(np.abs(mean_template.T[channel,:]))
#                         weights = weights/np.max(weights)
#                         low_values_indices = weights < 0.25  # Where values are low,
#                         weights[low_values_indices] = 0      # make the weight 0
#                         (xpos,ypos)=np.average(site_positions,axis=0,weights=weights)
#                     else:
#                         mean_template,xpos,ypos = (np.nan,np.nan,np.nan)
#                     self.spiketime_information[str(unit)]['template'] = mean_template
#                     self.spiketime_information[str(unit)]['xpos'] = xpos
#                     self.spiketime_information[str(unit)]['ypos'] = ypos - offset
                    
                    
#         ##################################################################################################################
#         if 'name' in kwargs.keys():
#             name = kwargs['name']
#         else:
#             name = 'extracellular electrophysiology data'
#         self.mod = self.create_module(name)
        
#         #add clustering
#         clustering = self.mod.create_interface("Clustering")
#         clustering.set_clusters(self.all_spiketimes,self.all_cluster_numbers,np.zeros(len(self.all_cluster_numbers)))
#         clustering.finalize()
        
#         #add clustering_waveforms
#         #self.add_clustering_waveforms()
        
#         #add unit times
#         self.add_unit_times()


        
#         self.mod.finalize()
#         ##################################################################################################################
        
#     def add_unit_times(self,**kwargs):
#         unit_times = self.mod.create_interface("UnitTimes")
#         for unit_name in self.spiketime_information:
#             unit_times.add_unit(unit_name = unit_name, 
#                                 unit_times = np.sort(self.spiketime_information[unit_name]['spike_times']),
#                                 description = "All spiketimes are in SI units (seconds)",
#                                 source = "Data spike-sorted by: "+self.meta['user']+' using '+self.meta['sort_method'])
#             # also add waveform information, if it is avaialble, which for right now is just with phy-template
#             # now also with kilosort!
#             if 'phy-template' in  self.meta['sort_method'] or 'kilosort' in  self.meta['sort_method']:
#                 unit_times.append_unit_data(unit_name,'template',self.spiketime_information[unit_name]['template'])
#                 unit_times.append_unit_data(unit_name,'xpos',self.spiketime_information[unit_name]['xpos'])
#                 unit_times.append_unit_data(unit_name,'ypos',self.spiketime_information[unit_name]['ypos'])

#         unit_times.finalize()
        



        
#     def add_clustering_waveforms(self,**kwargs):
#         clustering_waveforms = self.mod.create_interface("ClusterWaveforms")
#         for unit_name in self.spiketime_information:
#             clustering_waveforms.add_waveform(cluster = unit_name,
#                                               waveform_mean = self.spikes_data[unit_name]['waveform'],
#                                               waveform_sd = self.spikes_data[unit_name]['waveform_sd'])
#         clustering_waveforms.finalize()
    
    
#     #add stimulus timing information to the ax        
#     def add_stimulus_information(self,timestamps,data,start_time=0.0,source='dome',name='visual stimulus - generic',**kwargs):
#         abstract = self.create_timeseries("AbstractFeatureSeries",name, "stimulus")
        
#         if 'features' in kwargs.keys():
#             if 'features_units' in kwargs.keys():
#                 abstract.set_features(kwargs['features'],kwargs['features_units'])
#             else:
#                 self.fatal_error("features_units is required when features is used")
#         abstract.set_data(data)
#         abstract.set_time(timestamps+start_time)
#         abstract.set_source(source)

#         abstract.finalize()

#     #add stimulus timing information to the ax        
#     def add_stimulus_template(self,data,times=np.array([0]),start_time=0.0,source='dome',name='visual stimulus - generic',**kwargs):
#         abstract = self.create_timeseries("AbstractFeatureSeries",name, "template")
        
#         if 'features' in kwargs.keys():
#             if 'features_units' in kwargs.keys():
#                 abstract.set_features(kwargs['features'],kwargs['features_units'])
#             else:
#                 self.fatal_error("features_units is required when features is used")
#         abstract.set_data(data)
#         abstract.set_source(source)
#         abstract.set_time(times+start_time)
#         abstract.finalize()
    
#     def add_eyedata(self,data,times,name='righteye',start_time=0.0):
#         self.eye_mod = self.create_module(name)
#         pupil_area = self.eye_mod.create_interface("PupilTracking")
#         pupil_area_ts = self.create_timeseries('TimeSeries','PupilArea')
#         pupil_area_ts.set_data(data['pupil_area'],conversion=np.array([1]).astype(np.float32)[0],resolution=np.array([1]).astype(np.float32)[0],unit='pixels')
#         pupil_area_ts.set_time(times+start_time)
#         pupil_area.add_timeseries(pupil_area_ts)
#         pupil_area_ts.finalize()
#         pupil_area.finalize()
#         eye_position = self.eye_mod.create_interface("EyeTracking")
#         eye_position_ts = self.create_timeseries('SpatialSeries','EyePositions')
#         eye_position_ts.set_data(data['pupil_positions'],conversion=np.array([1]).astype(np.float32)[0],resolution=np.array([1]).astype(np.float32)[0],unit='pixels')
#         eye_position_ts.set_value('reference_frame',['[0,0]'])
#         eye_position_ts.set_time(times+start_time)
#         eye_position.add_timeseries(eye_position_ts)
#         eye_position_ts.finalize()
#         eye_position.finalize()
        
#         self.eye_mod.finalize()
        
#     def add_to_general(self,data,name='general'):
#         self.set_metadata(name,data)
        
def load_phy_template(path,cluster_file='KS2',site_positions = option234_positions, **kwargs):
# load spike data that has been manually sorted with the phy-template GUI
# the site_positions should contain coordinates of the channels in probe space. for example, in um on the face of the probe
# returns a dictionary of 'good' units, each of which includes:
#	times: spike times, in seconds
#	template: template used for matching
#	ypos: y position on the probe, calculated from the template. requires an accurate site_positions. averages template from 100 spikes.
#	xpos: x position on the probe, calcualted from the template. requires an accurate site_positions. averages template from 100 spikes.
    clusters = np.load(open(os.path.join(path,'spike_clusters.npy'),'rb'))
    spikes = np.load(open(os.path.join(path,'spike_times.npy'),'rb'))
    spike_templates = np.load(open(os.path.join(path,'spike_templates.npy'),'rb'))
    templates = np.load(open(os.path.join(path,'templates.npy'),'rb'))
    cluster_id,KSlabel,KSamplitude,KScontamination = [],[],[],[]
    [KSlabel.append(row) for row in csv.reader(open(os.path.join(path,'cluster_KSLabel.tsv')))]
    [KSamplitude.append(row) for row in csv.reader(open(os.path.join(path,'cluster_Amplitude.tsv')))]
    [KScontamination.append(row) for row in csv.reader(open(os.path.join(path,'cluster_ContamPct.tsv')))]
    if os.path.isfile(os.path.join(path,'cluster_group.tsv')):
        # cluster_id = [row for row in csv.reader(open(os.path.join(path,'cluster_group.tsv')))][1:]
        [cluster_id.append(row) for row in csv.reader(open(os.path.join(path,'cluster_group.tsv')))]
    else:
        if os.path.isfile(os.path.join(path,'cluster_groups.csv')):
            # cluster_id = [row for row in csv.reader(open(os.path.join(path,'cluster_groups.csv')))][1:]
            [cluster_id.append(row) for row in csv.reader(open(os.path.join(path,'cluster_groups.csv')))]
        else: print('cant find cluster groups, either .tsv or .csv')
    if 'sampling_rate' in kwargs.keys():
        samplingrate = kwargs['sampling_rate']
    else:
        samplingrate =30000.
        # print('no sampling rate specified, using default of 30kHz')
        
    units = {}
    for i in np.arange(1,np.shape(cluster_id)[0]):
        unit = int(cluster_id[i][0].split('\t')[0])
        units[str(unit)] = {}
        
        #get the unit spike times
        units[str(unit)]['samples'] = spikes[np.where(clusters==unit)].flatten()
        units[str(unit)]['times'] = spikes[np.where(clusters==unit)]/samplingrate
        units[str(unit)]['times'] = units[str(unit)]['times'].flatten()
        
        #get the mean template used for this unit
        all_templates = spike_templates[np.where(clusters==unit)].flatten()
        n_templates_to_subsample = 100
        random_subsample_of_templates = templates[all_templates[np.array(np.random.rand(n_templates_to_subsample)*all_templates.shape[0]).astype(int)]]
        mean_template = np.mean(random_subsample_of_templates,axis=0)
        units[str(unit)]['template'] = mean_template
        
        #take a weighted average of the site_positions, where the weights is the absolute value of the template for that channel
        #this gets us the x and y positions of the unit on the probe.
        # print(mean_template.T.shape)
        weights = np.zeros(site_positions.shape)
        for channel in range(mean_template.T.shape[0]):
            weights[channel,:]=np.trapz(np.abs(mean_template.T[channel,:]))
        weights = weights/np.max(weights)
        low_values_indices = weights < 0.25  # Where values are low,
        weights[low_values_indices] = 0      # make the weight 0
        (xpos,ypos)=np.average(site_positions,axis=0,weights=weights)
        units[str(unit)]['waveform_weights'] = weights
        units[str(unit)]['xpos'] = xpos
        units[str(unit)]['ypos'] = ypos #- site_positions[-1][1]
        units[str(unit)]['label'] =  cluster_id[i][0].split('\t')[1]
        units[str(unit)]['KSlabel'] =  KSlabel[i][0].split('\t')[1]
        units[str(unit)]['KSamplitude'] =  KSamplitude[i][0].split('\t')[1]
        units[str(unit)]['KScontamination'] =  KScontamination[i][0].split('\t')[1]
    return units

def df_from_phy(path,site_positions = option234_positions,**kwargs):
    nwb_data = load_phy_template(path,site_positions)
    #structures is a dictionary that defines the bounds of the structure e.g.:{'v1':(0,850), 'hpc':(850,2000)}
    mouse = [];experiment=[];cell = [];ypos = [];xpos = [];waveform=[];template=[];structure=[];times=[]
    index = []; count = 1
    nwb_id = [];probe_id=[]
    depth=[];#print(list(nwb_data.keys()));print(list(nwb_data['processing'].keys()));
    if 'probe' in kwargs.keys():
        for probe in list(nwb_data['processing'].keys()):
            if 'UnitTimes' in list(nwb_data['processing'][probe].keys()): 
                for i,u in enumerate(list(nwb_data['processing'][probe]['UnitTimes'].keys())):
                    if u != 'unit_list':
                        nwb_id.append(nwbid)
                        probe_id.append(probe)
                        index.append(count);count+=1
                        mouse.append(str(np.array(nwb_data.get('identifier'))))
                        experiment.append(1)
                        cell.append(u)
                        times.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['times']));# print(list(nwb_data['processing'][probe]['UnitTimes'][u].keys()))
                        if 'ypos' in list(nwb_data['processing'][probe]['UnitTimes'][u].keys()):
                            ypos.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['ypos']))
                            has_ypos = True
                        else:
                            ypos.append(None)
                            has_ypos = False				
                        if 'depth' in list(nwb_data['processing'][probe]['UnitTimes'][u].keys()):
                            depth.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['depth']))
                        else:
                            if has_ypos:
                                depth.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['ypos']))
                            else:
                                depth.append(None)
                        if 'xpos' in list(nwb_data['processing'][probe]['UnitTimes'][u].keys()):
                            xpos.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['xpos']))
                            has_xpos = True
                        else:
                            xpos.append(None)
                            has_xpos = False
                        template.append(np.array(nwb_data['processing'][probe]['UnitTimes'][u]['template']))
                        waveform.append(get_peak_waveform_from_template(template[-1]))
                        if not structures == None:
                            structur = None
                            for struct, bounds in structures.iteritems():
                                if ypos[-1] > bounds[0] and ypos[-1]< bounds[1] :
                                    structur=struct
                        else:
                            structur = None
                        structure.append(structur)
    df = pd.DataFrame(index=index)
    df = df.fillna(np.nan)
    df['nwb_id'] = nwb_id
    df['mouse'] = mouse
    df['experiment'] = experiment
    df['probe'] = probe_id
    df['structure'] = structure
    df['cell'] = cell
    df['times'] = times
    df['ypos'] = ypos
    df['xpos'] = xpos
    df['depth'] = depth
    df['waveform'] = waveform
    df['template'] = template
    return df

def load_unit_data(recording_path, probe_depth = 3840, site_positions = option234_positions, 
                   probe_name=None, spikes_filename = 'spike_secs.npy', aligned=True, df=True, **kwargs):
# Inputs:
# Outputs:
    if probe_name == None: probe_name = recording_path
    #Get individual folders for each probe
    unit_times=[]
    if aligned == False:
#         if not sampling_rate:
#             imec_meta = readAPMeta(recording_path+'\\') #extract meta file
#             sampRate = float(imec_meta['imSampRate']) #get sampling rate (Hz)
#         else:
	    if 'sampling_rate' in kwargs.keys():
		    sampRate  = float(kwargs['sampling_rate'])
	    else:
		    sampRate=30000
	    spike_times = np.ndarray.flatten(np.load(os.path.join(recording_path, 'spike_times.npy')))/sampRate
    else:
        spike_times = np.ndarray.flatten(np.load(os.path.join(recording_path, spikes_filename)))

    cluster_info = pd.read_csv(os.path.join(recording_path, 'cluster_info.tsv'), '\t')
    if cluster_info.keys()[0]=='cluster_id':
        cluster_info = cluster_info.rename(columns={'cluster_id':'id'})
    spike_clusters = np.ndarray.flatten(np.load(os.path.join(recording_path, 'spike_clusters.npy')))
    spike_templates = np.load(open(os.path.join(recording_path,'spike_templates.npy'),'rb'))
    templates = np.load(open(os.path.join(recording_path,'templates.npy'),'rb'))
    amplitudes = np.load(open(os.path.join(recording_path,'amplitudes.npy'),'rb'))
    weights = np.zeros(site_positions.shape)

    #Generate Unit Times Table
    for index, unitID in enumerate(cluster_info['id'].values):
        #get mean template used for each unit
        all_templates = spike_templates[np.where(spike_clusters==unitID)].flatten()
        n_templates_to_subsample = 100
        random_subsample_of_templates = templates[all_templates[np.array(np.random.rand(n_templates_to_subsample)*all_templates.shape[0]).astype(int)]]
        mean_template = np.mean(random_subsample_of_templates,axis=0)

        #take a weighted average of the site_positions, where the weights is the absolute value of the template for that channel
        #this gets us the x and y positions of the unit on the probe.
        for channel in range(mean_template.T.shape[0]):
            weights[channel,:]=np.trapz(np.abs(mean_template.T[channel,:]))
        weights = weights/np.max(weights)
        low_values_indices = weights < 0.25  # Where values are low,
        weights[low_values_indices] = 0      # make the weight 0
        (xpos,zpos)=np.average(site_positions,axis=0,weights=weights)

        unit_times.append({'probe':probe_name,
                           'unit_id': unitID,
                           'group': cluster_info.group[index],
#                                'depth':cluster_info.depth[index],
                           'depth': (zpos-3840)+probe_depth,
                           'xpos': xpos,
                           'zpos': zpos,
                           'no_spikes': cluster_info.n_spikes[index], 
                           'KSlabel': cluster_info['KSLabel'][index],
                           'KSamplitude':cluster_info.Amplitude[index],
                           'KScontamination': cluster_info.ContamPct[index],
                           'template': mean_template,
                           'waveform_weights': weights,
                           'amplitudes': amplitudes[:,0][spike_clusters==unitID],
                           'times': spike_times[spike_clusters == unitID],
                            })
    if df == True:        
        unit_data = pd.DataFrame(unit_times)
        #Remove clusters with no associated spike times left over from Phy
        for i,j in enumerate(unit_data.times):
            if len(unit_data.times[i])==0:
                unit_data.times[i]='empty'
        unit_times = unit_data[unit_data.times!='empty']
        return(unit_times)
    else:
        return(unit_times)

def multi_load_unit_data(recording_folder,probe_names=['A','B','C','D'],probe_depths=[3840,3840,3840,3840],spikes_filename = 'spike_secs.npy', aligned=True):
    folder_paths = glob.glob(os.path.join(recording_folder,'*imec*'))
    return pd.concat([load_unit_data(folder,probe_name=probe_names[i],probe_depth=probe_depths[i],spikes_filename = spikes_filename, aligned=True,df=True) for i,folder in enumerate(folder_paths)],ignore_index=True)



import pickle as pkl
import numpy as np
import datetime as dt
import pandas as pd
import os, h5py, json,glob
import matplotlib.pyplot as plt
from tqdm import tqdm

#Convert meta file into dictionary 
#Adapted from readSGLX.py package from SpikeGLX
def readAPMeta(bin_path):
    metaPath = glob.glob(os.path.join(bin_path,'*ap.meta'))[0]
    metaName = os.path.basename(metaPath)
    metaDict = {}
    if os.path.isfile(metaPath):
        # print("meta file present")
        with open(metaPath) as f:
            mdatList = f.read().splitlines()
            # convert the list entries into key value pairs
            for m in mdatList:
                csList = m.split(sep='=')
                if csList[0][0] == '~':
                    currKey = csList[0][1:len(csList[0])]
                else:
                    currKey = csList[0]
                metaDict.update({currKey: csList[1]})
    else:
        print("no meta file")
    return(metaDict)

def readNIMeta(bin_path):
    print(bin_path)
    metaPath = glob.glob(os.path.join(bin_path,'*nidq.meta'))[0] #glob.glob(bin_path+'*nidq.meta')[0]
    metaName = os.path.basename(metaPath)
    metaDict = {}
    if os.path.isfile(metaPath):
        # print("meta file present")
        with open(metaPath) as f:
            mdatList = f.read().splitlines()
            # convert the list entries into key value pairs
            for m in mdatList:
                csList = m.split(sep='=')
                if csList[0][0] == '~':
                    currKey = csList[0][1:len(csList[0])]
                else:
                    currKey = csList[0]
                metaDict.update({currKey: csList[1]})
    else:
        print("no meta file")
    return(metaDict)

#Takes the .bin file output from your SpikeGLX recording and outputs a plot of the individual 
# digital lines over time 
def parse_ni_digital(bin_path, seconds=True):
    print('Sit back. This is going to take a while!')
    #Memory map the bin file and parse into binary lines
    mm = np.memmap(glob.glob(os.path.join(bin_path, '*bin'))[0],dtype=np.int16) #glob.glob(bin_path+'*bin')[0],dtype=np.int16)
    digital_words = mm[8::9]
    
    #Extract the number of digital channels from the meta file
    meta = readNIMeta(bin_path)
    nchans = meta['niXDChans1']
    ncs = nchans.split(":")
    nChans = int(ncs[1])-int(ncs[0])+1
    
    #go through each timepoint and figure out if each line has switched states (high --> low or low --> high). 
    #the output will be two dictionaries digital_lines_rising and digital_lines_falling, 
    #which have each sample where the transition happened
    num_digital_channels=nChans 
    digital_lines_rising = {}
    digital_lines_falling = {}
    for i in tqdm(range(digital_words.shape[0])[::10]): #note that this downsamples by factor 10, to 100kHz
        if i==0:
            state_previous_sample = '{0:08b}'.format(digital_words[i]) #Parse digital words into binary lines. 1=High, 0=Low
            for line in range(num_digital_channels):
                digital_lines_rising['D'+str(line)] = [] #initialize empty list
                digital_lines_falling['D'+str(line)] = [] #initialize empty list
        else:
            state_this_sample = '{0:08b}'.format(digital_words[i]) #Parse digital words into binary lines. 1=High, 0=Low
            changes = [j for j in range(len(state_previous_sample)) if state_previous_sample[j] != state_this_sample[j]]
            for line in changes:   
                    if state_this_sample[line] == '1':
                        digital_lines_rising['D'+str(line)].extend([i*10]) #note that this scales back up to 1MHz sampling rate 
                    else:
                        digital_lines_falling['D'+str(line)].extend([i*10])  #note that this scales back up to 1MHz sampling rate 
            state_previous_sample=state_this_sample #update sample
            
    #Reorder the lines manually because they were set up backwards above
    #TODO: fix the above loop to put things in the right order the first time
    digital_lines_rising2 = {}
    digital_lines_falling2 = {}
    for i,key in enumerate(digital_lines_rising.keys()):
        digital_lines_rising2['D'+str(7-i)]=digital_lines_rising[key]
        digital_lines_falling2['D'+str(7-i)]=digital_lines_falling[key]
    digital_lines_rising = digital_lines_rising2
    digital_lines_falling = digital_lines_falling2
    if seconds==False:
        return(digital_lines_rising, digital_lines_falling) 

    #Convert from sample times to seconds
    if seconds==True:
        for line in digital_lines_rising.keys():
            digital_lines_rising[line] = np.array(digital_lines_rising[line])/float(meta['niSampRate'])
        for line in digital_lines_falling.keys():
            digital_lines_falling[line] = np.array(digital_lines_falling[line])/float(meta['niSampRate'])
        return(digital_lines_rising, digital_lines_falling) 

#plot the output of `sglx_nidaq`
def sglx_nidaq_plot(digital_lines_rising):
        for i,line in enumerate(digital_lines_rising.keys()):
            plt.plot(np.array(digital_lines_rising[line]),np.ones(len(digital_lines_rising[line]))*(7-i),'-o',label=line)
            plt.legend(loc='lower right')

#autoparse the output of sglx_nidaq into a dictionary of timestamps 
#SPECIFIC TO COLOR_POPULATION.PY
#requires 2 lists 1)stimulus names for line D1 in order 2)number of frames per stimulus in order
def cpop_autoparse(nidaq_dlr, d1frames, d1stims, d2frames, d2stims):
    #NIDAQ digital lines rising data will either come from a saved pkl file or a dictionary output of spikeglx_nidaq
    if isinstance(nidaq_dlr, str):
        with open(nidaq_dlr, 'rb') as a:
            nidaq = pkl.load(a)
    else:
        nidaq = nidaq_dlr
    #isolate the keys containing timestaps from lines D1 and D3
    d1 = np.array(nidaq['D1'])/1e7#ONLY FOR TESTING FIX BEFORE USING
    d2 = np.array(nidaq['D2'])/1e7#ONLY FOR TESTING FIX BEFORE USING
    
    #parse into dictionary
    stop=0
    stimulus_timestamps={}
    for i,j in zip(d1stims, enumerate(d1frames)):
        j=int(j[0])
        start = stop
        stop = start+d1frames[j]
        stim_ts = np.array(d1[start:stop])      
        stimulus_timestamps.update({str(i): stim_ts})

    for i,j in zip(d2stims, enumerate(d2frames)):
            j=int(j[0])
            start = stop
            stop = start+d2frames[j]
            stim_ts = np.array(d2[start:stop])      
            stimulus_timestamps.update({str(i): stim_ts})
 
    return(stimulus_timestamps)

#Takes pkl output from stimulus timestamps from cpop_autoparse to make 
#a dictionary with the stimulus data and timestamps. Needed to be a dictionary because of 3d arrays in stim data
#will dump pkl of dictionary in output path if save = true
def cpop_mtx_int(matrix_pkl, timestamps, output_path, save=True):
    if type(matrix_pkl)==dict:
        for key in matrix_pkl.keys():
            if key=='stackG':
                color_matrix_green = {'times':timestamps, 'frames':matrix_pkl[key]}
                if save==True:
                    pkl.dump(color_matrix_green,open(os.path.join(output_path, 'color_matrix_green.pkl'),'wb'))
            elif key=='stackB':
                color_matrix_uv = {'times':timestamps, 'frames':matrix_pkl[key]}
                if save==True:
                    pkl.dump(color_matrix_uv,open(os.path.join(output_path, 'color_matrix_uv.pkl'),'wb'))
        return(color_matrix_green, color_matrix_uv)
    else:
        highspeed_data = {'times':timestamps, 'frames':matrix_pkl}
        if save==True:
            pkl.dump(highspeed_data,open(os.path.join(output_path, 'highspeed.pkl'),'wb'))
        return(highspeed_data)

#returns a dataframe for each gratings stimulus
#unable to save because df.to_json doesnt work in function
#TODO: incorporate errors and dropped frames
#if youre working with gratings_color.py, ensure gratings_color=True
def cpop_gratings_int(gratings_pkl,timestamps,color_gratings=False):  
    if color_gratings==True:    
        gratings_df = pd.DataFrame(gratings_pkl['bgsweeptable'], 
                                   columns = ['Contrast', 'PosY', 'TF', 'SF', 'Phase', 'PosX', 'Ori', 'Color'])
        gratings_df['frame_no'] = gratings_df.index
        frameno_df = pd.DataFrame(gratings_pkl['bgsweeporder'], columns = ['frame_no'])
        gratings_df1 = pd.merge(left = frameno_df, 
                          right = gratings_df, 
                          on = 'frame_no', 
                          sort = False, 
                          how = 'left')
        gratings_df1[['R', 'G', 'B']] = pd.DataFrame(gratings_df1['Color'].tolist(), index=gratings_df1.index)   
        gratings_df2 = gratings_df1.drop(columns=['Color'])
        gratings_df2['times'] = timestamps
        return(gratings_df2)
        #if save==True:
            #filename = str(os.path.join(output_path, pkl_str+'.json'))
            #json.dump(gratings_df2, filename)
        return(gratings_df2)
    
    else:
        gratings_df = pd.DataFrame(gratings_pkl['bgsweeptable'], columns = ['contrast', 'posY', 'TF', 'SF', 'phase', 'posX', 'ori'])
        gratings_df['frame_no'] = gratings_df.index

        frameno_df = pd.DataFrame(gratings_pkl['bgsweeporder'], columns = ['frame_no'])

        gratings_df1 = pd.merge(left = frameno_df, 
                              right = gratings_df, 
                              on = 'frame_no', 
                              sort = False, 
                              how = 'left')
        gratings_df1['times'] = timestamps
        #print(gratings_df1.head())
        #if save==True:
            #filename = os.path.join(output_path, pkl_str+'.json')
            #gratings_df1.to_json()
        return(gratings_df1)

#Returns a dataframe for the scene flicker stimului like the matrix and gratings functions above
def cpop_scene_int(scene_pkl,timestamps):
    scene_files = []
    for i in range(len(scene_pkl['imagefiles'])):
        scene_files.append(os.path.basename(scene_pkl['imagefiles'][i])),

    scene_df = pd.DataFrame(scene_files, columns = ['Image File'])
    scene_df['frame_no'] = scene_df.index
    frameno_df = pd.DataFrame(scene_pkl['bgsweeporder'], columns = ['frame_no'])
    scene_df1 = pd.merge(left = frameno_df, 
                          right = scene_df, 
                          on = 'frame_no', 
                          sort = False, 
                          how = 'left')
    scene_df1.astype(dtype='S10')
    scene_df1['times'] = timestamps
    return(scene_df1)

#returns a data frame which contains all spike time data as well as the relevant info contained in cluster_info
#Highly recommend exporting the dataframe to .json
#dataPath is the path holding ALL probe folders
def unitTimes(dataPath,**sampling_rate):
    #get individual folders for each probe
    folder_paths = []
    imec0_path = glob.glob(dataPath+'*imec0')
    if len(imec0_path)!=0:
        folder_paths.append(imec0_path)
    imec1_path = glob.glob(dataPath+'*imec1')
    if len(imec1_path)!=0:
        folder_paths.append(imec1_path)        
    imec2_path = glob.glob(dataPath+'*imec2')
    if len(imec2_path)!=0:
        folder_paths.append(imec2_path)
    imec3_path = glob.glob(dataPath+'*imec3')
    if len(imec3_path)!=0:
        folder_paths.append(imec3_path)
    #print(folder_paths)    
    #get spike times and cluster groups
    unit_times = []
    for i, folder in enumerate(tqdm(folder_paths)):
        probe_names = ['imec0', 'imec1', 'imec2', 'imec3']
        # if not sampling_rate:
        #     imec_meta = readMeta(folder[0]+'\\') #extract meta file
        #     sampRate = float(imec_meta['imSampRate']) #get sampling rate (Hz)
        # else:
        #     sampRate  = float(sampling_rate['sampling_rate'])
        #cluster_groups = pd.read_csv(os.path.join(folder[0], 'cluster_group.tsv'), '\t') #redundant data found in cluster_info
        cluster_info = pd.read_csv(os.path.join(folder[0], 'cluster_info.tsv'), '\t')
        # spike_times = np.ndarray.flatten(np.load(os.path.join(folder[0], 'spike_secs.npy')))
        # spike_seconds = np.ndarray.flatten(spike_times/sampRate) #convert spike times to seconds from samples
        spike_seconds = np.ndarray.flatten(np.load(os.path.join(folder[0], 'spike_secs.npy')))
        spike_clusters = np.ndarray.flatten(np.load(os.path.join(folder[0], 'spike_clusters.npy')))
    
        #Generate Unit Times Table
        for index, unitID in enumerate(cluster_info['id'].values):
            unit_times.append({'probe':probe_names[i],
                               'unit_id': unitID,
                               'group': cluster_info.group[index],
                               'depth':cluster_info.depth[index],
                               'no_spikes': cluster_info.n_spikes[index], 
                               'amplitude':cluster_info.Amplitude[index],
                               'times': spike_seconds[spike_clusters == unitID],
                                })
    unit_data = pd.DataFrame(unit_times)
    #Remove clusters with no associated spike times left over from Phy
    for i,j in enumerate(unit_data.times):
        if len(unit_data.times[i])==0:
            unit_data.times[i]='empty'
    unit_times = unit_data[unit_data.times!='empty']
    return(unit_times)


def cleanAxes(ax,bottomLabels=False,leftLabels=False,rightLabels=False,topLabels=False,total=False):
    ax.tick_params(axis='both',labelsize=10)
    ax.spines['top'].set_visible(False);
    ax.yaxis.set_ticks_position('left');
    ax.spines['right'].set_visible(False);
    ax.xaxis.set_ticks_position('bottom')
    if not bottomLabels or topLabels:
        ax.set_xticklabels([])
    if not leftLabels or rightLabels:
        ax.set_yticklabels([])
    if rightLabels:
        ax.spines['right'].set_visible(True);
        ax.spines['left'].set_visible(False);
        ax.yaxis.set_ticks_position('right');
    if total:
        ax.set_frame_on(False);
        ax.set_xticklabels('',visible=False);
        ax.set_xticks([]);
        ax.set_yticklabels('',visible=False);
        ax.set_yticks([])


import numpy as np
import random
from scipy import stats
from scipy.signal import boxcar,convolve,correlate,resample,argrelextrema
from scipy.cluster.vq import kmeans,kmeans2
from scipy.stats import pearsonr
from dlab import cleanAxes
from dlab import psth_and_raster as psth_

def smooth_boxcar(data,boxcar_size):
    smoothed = convolve(data,boxcar(int(boxcar_size)))/boxcar_size
    smoothed = smoothed[int(boxcar_size/2):len(data)+int(boxcar_size/2)]
    return smoothed

def moving_average(a, n=3) :
    ret = np.cumsum(a, dtype=float)
    ret[n:] = ret[n:] - ret[:-n]
    return ret[n - 1:] / n

def get_peak_waveform_from_template(template):
    max = 0
    ind=0
    peak = np.zeros(np.shape(template.T)[0])
    for i,wv in enumerate(template.T):
        if np.max(np.abs(wv)) > max:
            max = np.max(np.abs(wv))
            ind = i
            peak = wv
    return peak

def precision(spike_times,stimulus_times,boxcar_size = 5,precision_type='first',pre=0.,post=15.,binsize=0.01,threshold=0.05,find_events=True):
# precision - jitter of first spike, jitter of total spikes
    if find_events:
        smoothed,all_events,events = get_events(spike_times,stimulus_times,boxcar_size = boxcar_size,threshold=threshold,pre=pre,post=post,binsize=binsize)
    else:
        events = [0.]
    #print events
    #get spikes for each event
    event_precision = []
    for i,event in enumerate(events):
        all_spikes = []
        first_spikes = []
        for trial in stimulus_times:
            event_start = trial + event
            if i == len(events):
                if event[i+1] - event > 0.05:
                    event_end = trial + event[i+1]
                else:
                    event_end = trial + event + 0.05
            else:
                event_end = trial + event + 0.05
            indices = np.where((spike_times > event_start) & (spike_times < event_end))[0] 

            first_spike = spike_times[np.where(spike_times>event_start)[0][0]]-event_start
            #find jitter of first spikes
            if first_spike < .1: #arbitrary cutoff because we don't trust identified events that don't have a spike within 100 msec
                first_spikes.extend([first_spike])

            #find jitter of all spikes
            event_spikes = np.array(spike_times[indices])-event_start
            all_spikes.extend(event_spikes)
  
        if precision_type =='first':
            #event_precision.extend([np.median(np.array(first_spikes))])
            #event_precision.extend([np.median(np.array(first_spikes)-np.min(first_spikes))])
            all_event_spikes = np.sort(np.array(first_spikes))
        else:
            all_event_spikes = np.sort(np.array(all_spikes).flatten())
            
        #print all_event_spikes
        first_half = all_event_spikes[:np.where(all_event_spikes<np.median(all_event_spikes))[0][-1]]
        second_half = all_event_spikes[np.where(all_event_spikes>np.median(all_event_spikes))[0][0]:]
        event_precision.extend([np.median(second_half)-np.median(first_half)])
    
    event_precision = np.array(event_precision)
    event_precision = event_precision[~np.isnan(event_precision)] * 1000.
    
    return (np.mean(event_precision),np.std(event_precision),event_precision)

def get_binarized(spike_times,stimulus_times,pre=0.,post=15.,convolve=0.):
    bytrial = psth_.raster(spike_times,stimulus_times,pre=pre,post=post,timeDomain=True,output='data')
    binarized = []
    for trial in bytrial:
        binarized_ = np.zeros(int((post-pre)*1000))#use 1-msec bins
        for spike in trial:
            if spike > pre and spike < post:
                binarized_[int(np.floor(spike*1000))] = 1
        if convolve > 0.001 :
            binarized_=smooth_boxcar(binarized_,convolve)
        binarized.append(binarized_)   
    return binarized

def get_binned(spike_times,stimulus_times,binsize,pre=0.,post=15.,convolve=0.):
    bytrial = psth_.raster(spike_times,stimulus_times,pre=pre,post=post,timeDomain=True,output='data')
    # print(np.array(bytrial).shape())
    binarized = []
    for trial in bytrial:
        binarized_ = np.zeros(int((post-pre)*1000/(1000*binsize)+1))#use binsize msec bins
        for spike in trial:
            if spike > pre and spike < post:
                binarized_[int(np.floor((spike-pre)*1000/(1000*binsize)))] += 1
        if convolve > 0.001 :
            binarized_=smooth_boxcar(binarized_,convolve)
        binarized.append(binarized_)   
    return binarized

def reliability(spike_times,stimulus_times,binsize,pre=0.,post=15.):
#reliability - how reproducible a spike train is over trials, at the msec level
    binarized=get_binned(spike_times,stimulus_times,binsize,pre=pre,post=post)
    sum = 0
    for c,i in enumerate(binarized):
        for c2 in np.arange(c+1,np.shape(binarized)[0],1):
            j = binarized[c2][:]
            if np.sum(i) > 0 and np.sum(j) > 0:
                sum += np.inner(i,j) / np.inner(np.linalg.norm(i),np.linalg.norm(j))
            else:
                sum+=0

    #return (2 / float(np.shape(binarized)[0]) * (np.shape(binarized)[0] - 1)) * sum,binarized
    return sum / (np.shape(binarized)[0]/2.)#,binarized

def fano(spike_times,stimulus_times,pre=0.,post=15.,binsize=0.01,boxcar_size = 5,counting_window=0.3,threshold=0.2,by_event=False):
    if by_event:
        smoothed,all_events,events = get_events(spike_times,stimulus_times,boxcar_size = boxcar_size,threshold=threshold,pre=pre,post=post,binsize=binsize)
    else:
        events = [0]
    fanos=[]
    for i,event in enumerate(events):
        counts=[]
        for trial in stimulus_times:
            event_start = trial + event
            event_end = trial + event + counting_window
            indices = np.where((spike_times > event_start) & (spike_times < event_end))[0] 
            
            counts.append(len(indices))

        fanos.append(np.std(counts)**2 / np.mean(counts))
    return np.median(fanos),counts, fanos

def get_events(spike_times,stimulus_times,threshold=.05,boxcar_size = 15,pre=0.,post=15.,binsize=0.001):
    (edges,psth,variance) =  psth_.psth_line(spike_times,
                                             stimulus_times,
                                             pre=pre,post=post,binsize=binsize,output='p',timeDomain=True)
    numbins = int((post-pre) /  binsize)

    # first, find events:
    smoothed = smooth_boxcar(psth[:numbins],int(boxcar_size))
    minima = np.where(np.r_[True, smoothed[2:] > smoothed[:-2]] & np.r_[smoothed[:-2] > smoothed[2:], True]==True)[0]#from http://stackoverflow.com/questions/4624970/finding-local-maxima-minima-with-numpy-in-a-1d-numpy-array
   
    #print np.max(psth)
    threshold = threshold * np.max(psth)
    good_minima = []
    #print threshold
    for i,minimum in enumerate(minima[:-1]):
        if minima[i+1]*binsize - minima[i]*binsize > 0.3:
            num_bins_after_minimum = 0.3 / binsize
        else:
            num_bins_after_minimum = minima[i+1] - minima[i]
        try:
            if np.max(psth[minimum:minimum+num_bins_after_minimum]) > threshold:
                good_minima.extend([minimum])
        except:
            pass
            # print np.shape(psth)
            # print minimum
            # print num_bins_after_minimum
            # print threshold
            # return np.nan,np.nan,np.nan
            # 

    return smoothed,minima*binsize,np.array(good_minima)*binsize - (boxcar_size/2.)*binsize

def entropy(spike_times,stimulus_times,wordlength,binsize=0.001,pre=0.,post=15.):
    binarized=get_binarized(spike_times,stimulus_times,pre=pre,post=post)

    # create words of length wordlength 
    entropies_per_time=[]
    for t in range(len(binarized[0])):
        words = []
        for trial in binarized:
            if t<len(trial) - wordlength: #cutoff of the end of each trial because there aren't enough bins left to make the word
                word = trial[t:t+wordlength]
                words.append(word)

        #make a distribution of the words
        p = {}
        #find all the words that actually occured. and the frequency of their occurence
        #from http://stackoverflow.com/questions/16970982/find-unique-rows-in-numpy-array
        # and http://stackoverflow.com/questions/33235135/count-occurrences-of-arrays-in-multidimensional-arrays-in-python
        words = np.array(words)
        b = np.ascontiguousarray(words).view(np.dtype((np.void, words.dtype.itemsize * words.shape[1])))
        _, idx,counts = np.unique(b, return_index=True,return_counts=True)
        possible_words = words[idx]
        p = dict(zip([str(word) for word in possible_words],counts/float(np.shape(np.array(words))[0])))
        print('all the words that occured:'+str(np.shape(np.array(words))[0]))

        # calculate entropy as in reinagel and reid 2000; kumbhani et al., 2007
        sum = 0
        for word in possible_words:
            sum +=  p[str(word)] * np.log2(p[str(word)])
        H = (-1 / (wordlength*binsize)) * sum
        entropies_per_time.append(H)
        
    return np.mean(entropies_per_time)

def fano2(a):
# computes the Fano factor , variance**2/mean for an input matrix a with dimensions n x m, 
# where n is the number of trials and m is the number of bins in the response
    return np.std(a,axis=0)**2/np.mean(a,axis=0)

def ccmax(a):
# computes the cc_max for an input matrix a with dimensions n x m, 
# where n is the number of trials and m is the number of bins in the response
    ntrials = a.shape[0]
    corr_matrix = np.empty((ntrials,ntrials))
    for i in range(ntrials):
        for j in range(ntrials):
            r,p = pearsonr(a[i,:],a[j,:])
            corr_matrix[i,j] = r

    inds = np.triu_indices(ntrials, k=1)
    upper = corr_matrix[inds[0],inds[1]]
    return np.nanmean(upper)

def mutual_information(spike_times,stimulus_times,wordlength,binsize=0.001,pre=0.,post=15.,method='poisson'):
    indices = np.where((spike_times > stimulus_times[0]) & (spike_times < stimulus_times[-1]))[0] 
    if method == 'poisson':
        reference_spikes = possion_times(np.array(spike_times)[indices])
    if method == 'shift':
        reference_spikes = shifted_times(np.array(spike_times)[indices],stimulus_times)
    total = entropy(reference_spikes,stimulus_times,wordlength,binsize=0.001,pre=0.,post=15.)
    noise = entropy(np.array(spike_times),stimulus_times,wordlength,binsize=0.001,pre=0.,post=15.)
    return total,noise, total-noise, (total-noise) / (post - pre)

def z_value(spike_times,stimulus_times,binsize,pre=0.,post=15.,method='shift'):
#based on reinagel and reid, 2000
# their definition, sightly modified for easier notation here
# Z(binsize) = limI(L,binsize) - I(L=1,binsize)
# "The term I(L=1,binsize) represents the estimate of information rate that would be obtained on the approximation that time bins are statistically independent within the spike train."
    lim_I = mutual_information(spike_times,stimulus_times,wordlength,binsize=binsize,pre=0.,post=15.,method='poisson')
    I1 = mutual_information(spike_times,stimulus_times,1,binsize=binsize,pre=0.,post=15.,method='poisson')
    return z

def possion_times(spike_times):
#given an input spike train, make the times a rate-matched Poisson process
    rate = len(np.array(spike_times)) / (np.array(spike_times)[-1] - np.array(spike_times)[0])
    t = np.array(spike_times)[0]
    poiss = [t]
    for i in range(len(spike_times)-1):
        t+=random.expovariate(rate)
        poiss.append(t)
    return np.array(poiss)

def shuffled_times(spike_times,window):
#given an input spike train, shuffle the spike times preserving the structure within the window
    return spike_times

def shifted_times(spike_times,stimulus_times):
#given an input spike train and stimulus times, 
#shift the spike times in each interval by an random amount, with wrapping
    shifted_times = np.zeros(len(spike_times))
    for i,start in enumerate(stimulus_times[:-1]):
        indices = np.where((spike_times > start) & (spike_times <= stimulus_times[i+1]))[0] - 1
        times = spike_times[indices]
        offset = np.floor(np.random.rand() * (stimulus_times[i+1] - start))
        offset_times = times + offset + start - i*(stimulus_times[i+1] - start)
        wrapped_times = np.array([b if b < stimulus_times[i+1] else b-(stimulus_times[i+1]-start) for b in offset_times])
        #print str(start)+' '+str(offset)+' '+str(offset_times)+' '+str(wrapped_times)

        shifted_times[indices] = wrapped_times
    indices = np.where(spike_times > stimulus_times[-1])[0]
    times = spike_times[indices]
    offset = np.random.rand() * (spike_times[-1] - stimulus_times[-1])
    offset_times = times + offset
    wrapped_times = np.array([b if b < spike_times[-1] else b-(spike_times[-1]-stimulus_times[-1]) for b in offset_times])
    shifted_times[indices] = wrapped_times
    return np.sort(shifted_times) - stimulus_times[0]


# def Rjitter_pair(spike_times, other_spike times):
#     spearmanr(spike_times,other_spike,nan_policy='omit')
    


from pynwb import NWBHDF5IO
import h5py, glob, os
import numpy as np
from ellipse import LsqEllipse
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import seaborn as sns
import tqdm
from p_tqdm import p_map
import time
import pandas as pd
from pynwb import TimeSeries
from pynwb.behavior import PupilTracking
from hdmf.backends.hdf5.h5_utils import H5DataIO
from datetime import datetime
from dateutil.tz import tzlocal
from pynwb import NWBFile, TimeSeries, NWBHDF5IO, ProcessingModule
from pynwb.behavior import PupilTracking
from joblib import Parallel, delayed

def get_eye_frame_from_NWB(nwb,index):
    positions = ['DR','D','LD','L','RV','R','VL','V']
    x,y = [],[]
    for p in positions:
        pos = np.array(nwb.processing.get('behavior').get('PupilTracking')[p+'pupil_positions'].data)
        x.extend([pos[index,0]])
        y.extend([pos[index,1]])
    return x,y
def get_eye_frame(df,index):
    positions = ['DR','D','LD','L','RV','R','VL','V']
    x,y = [],[]
    for p in positions:
        x_ = df[p+'pupil_X'].values[index]
        y_ = df[p+'pupil_Y'].values[index]
        x.extend([x_])
        y.extend([y_])
    return x,y
def fit_ellipse(x,y):
    lsqe = LsqEllipse()
    reg = lsqe.fit( np.array(list(zip(x, y))))
    center, width, height, phi = reg.as_parameters()
    return center, width, height, phi 

def make_OKR_NWB(recording_folder,
                 experimenter = 'j_h',
                 experiment_description= 'Denman Lab (Hughes Lab collaboration) on Autobahn Therepeutics remyelination therapy.'):
    if len(glob.glob(os.path.join(recording_folder,'*resnet*'))) > 2: 
        print('making NWB for '+recording_folder)
        try:
            #load data
            df_stimulus_frame_times = pd.read_csv(os.path.join(recording_folder,'stimulus_frame_times.csv'))
            df_stimulus_params = pd.read_csv(os.path.join(recording_folder,'stimulus_params.csv'))
            df_stimulus_times = pd.read_csv(os.path.join(recording_folder,'stimulus_times.csv'),index_col=False)

            df_stimulus_times['Orientation']=df_stimulus_params
            df_stimulus_times['TemporalFrequency']=[float(a[1:]) for a in df_stimulus_times.Value.values]
            df_stimulus_times.drop('Value', axis=1, inplace=True)

            df_camera_frames = pd.read_csv(os.path.join(recording_folder,'camera_times.csv'),index_col=False,header=None)
            df_dlc = pd.read_csv(glob.glob(os.path.join(recording_folder,'camera*0.csv'))[0],skiprows=2)
            df_dlc.columns=['frame','Lpupil_X','Lpupil_Y','Lpupil_likelihood','LDpupil_X','LDpupil_Y','LDpupil_likelihood',
                            'Dpupil_X','Dpupil_Y','Dpupil_likelihood','DRpupil_X','DRpupil_Y','DRpupil_likelihood',
                        'Rpupil_X','Rpupil_Y','Rpupil_likelihood','RVpupil_X','RVpupil_Y','RVpupil_likelihood',
                            'Vpupil_X','Vpupil_Y','Vpupil_likelihood','VLpupil_X','VLpupil_Y','VLpupil_likelihood','IRLED_X','IRLED_Y','IRLED_likelihood']
            df_dlc_filtered = pd.read_csv(glob.glob(os.path.join(recording_folder,'camera*_filtered.csv'))[0],skiprows=2)
            df_dlc_filtered.columns=['frame','Lpupil_X','Lpupil_Y','Lpupil_likelihood','LDpupil_X','LDpupil_Y','LDpupil_likelihood',
                            'Dpupil_X','Dpupil_Y','Dpupil_likelihood','DRpupil_X','DRpupil_Y','DRpupil_likelihood',
                        'Rpupil_X','Rpupil_Y','Rpupil_likelihood','RVpupil_X','RVpupil_Y','RVpupil_likelihood',
                            'Vpupil_X','Vpupil_Y','Vpupil_likelihood','VLpupil_X','VLpupil_Y','VLpupil_likelihood','IRLED_X','IRLED_Y','IRLED_likelihood']


            #create file
            nwbfile = NWBFile('Autobahn_myelin', 
                            recording_folder, 
                            datetime.now(tzlocal()),
                            experimenter=experimenter,
                            lab='Denman Lab',
                            institution='University of Colorado',
                            experiment_description=experiment_description,
                            session_id=os.path.basename(recording_folder))
            behavior_module = ProcessingModule('behavior', 'behavior module')
            nwbfile.add_processing_module(behavior_module)

            #add trials
            nwbfile.add_epoch(df_stimulus_times.Timestamp.iloc[0], 
                            df_stimulus_times.Timestamp.iloc[-1],
                            ['full_field_gratings'])
            nwbfile.add_trial_column('Orientation', 'the grating orientation')
            nwbfile.add_trial_column('TemporalFrequency', 'the grating temporal frequency')
            for i,row in df_stimulus_times.iterrows():
                nwbfile.add_trial(start_time=row.Timestamp,
                                stop_time=row.Timestamp+np.mean(np.diff(df_stimulus_times.Timestamp.values)),
                                Orientation=row.Orientation,
                                TemporalFrequency=row.TemporalFrequency)

            #add eye tracking
            eye_timestamps = df_camera_frames[0].values/1e9 - df_camera_frames[0].values[0]/1e9
            pupil = TimeSeries(
                name='IR_position',
                timestamps=eye_timestamps,
                data=np.ravel(np.column_stack((df_dlc.IRLED_X.values,df_dlc.IRLED_Y.values))).reshape(-1,2),
                unit='arb. unit',
                description='Features extracted from the video of the right eye.',
                comments='the IR LED position'
            )
            pupil_track = PupilTracking(pupil)
            for p in ['L','LD','D','DR','R','RV','V','VL']:
                eye_xy = TimeSeries(
                    name=p+'pupil_positions',
                    timestamps=eye_timestamps,
                    data=np.ravel(np.column_stack((df_dlc[p+'pupil_X'].values,df_dlc[p+'pupil_Y'].values))).reshape(-1,2),
                    unit='arb. unit',
                    description='Features extracted from the video of the right eye.',
                    comments='The 2D position of the center of this marker in the video '
                            'frame. This is not registered to degrees visual angle, but '
                            'could be used to detect saccades or other changes in eye position.'
                )
                pupil_track.add_timeseries(eye_xy)

                eye_xy_filtered = TimeSeries(
                    name=p+'pupil_positions_filtered',
                    timestamps=eye_timestamps,
                    data=np.ravel(np.column_stack((df_dlc_filtered[p+'pupil_X'].values,df_dlc_filtered[p+'pupil_Y'].values))).reshape(-1,2),
                    unit='arb. unit',
                    description='Features extracted from the video of the right eye.',
                    comments='The 2D position of the center of this marker in the video '
                            'frame. This is not registered to degrees visual angle, but '
                            'could be used to detect saccades or other changes in eye position.'   
                )
                pupil_track.add_timeseries(eye_xy_filtered)


            #do ellipse fitting
            num_frames = np.ravel(np.column_stack((df_dlc.IRLED_X.values,df_dlc.IRLED_Y.values))).reshape(-1,2).shape[0]
            center = np.zeros((num_frames,2))
            width= np.zeros(num_frames)
            height= np.zeros(num_frames)
            phi = np.zeros(num_frames)
            xdeg= np.zeros(num_frames)
            ydeg= np.zeros(num_frames)

            def do_ellipse_fit(i):
                try:
                    x,y = get_eye_frame(df_dlc_filtered,i)
                    (center,width,height,phi) = fit_ellipse(x,y)
                    return (i,center,width,height,phi)
                except: return (i,np.nan,np.nan,np.nan,np.nan)

            # pool = Pool()
            inds = np.arange(num_frames)
            res=p_map(do_ellipse_fit,inds,num_cpus=7)

            for i,cen, w, h, p in res:
                (center[i,:],width[i],height[i],phi[i]) = cen, w, h, p 
                r = (height[i]+width[i])/2.
                ydeg[i]=np.rad2deg(np.arccos(height[i]/r))
                xdeg[i]=np.rad2deg(np.arccos(r/width[i]))
            
            # pupil = nwb.processing['behavior'].data_interfaces['PupilTracking']

            center_xy = TimeSeries(
                name='pupil_center_positions',
                timestamps=eye_timestamps,
                data=np.array([center[:,0],center[:,1]]),
                unit='pixel',
                description='Features extracted from the video of the right eye.',
                comments='The 2D position of the center of the pupil in the video '
                        'frame. This is not registered to degrees visual angle, but '
                        'could be used to detect saccades or other changes in eye position.'
            )

            width_ = TimeSeries(
            name='pupil_width',
            timestamps=eye_timestamps,
            data=width,
            unit='pixel',
            description='Features extracted from the video of the right eye.',
            comments='The width of the pupil ellipse in the video '
                    'frame. This is not registered to degrees visual angle, but '
                    'could be used to detect saccades or other changes in eye position.'
            )

            height_ = TimeSeries(
            name='pupil_height',
            timestamps=eye_timestamps,
            data=height,
            unit='pixel',
            description='Features extracted from the video of the right eye.',
            comments='The height of the pupil ellipse in the video '
                    'frame. This is not registered to degrees visual angle, but '
                    'could be used to detect saccades or other changes in eye position.'
            )

            phi_ = TimeSeries(
            name='pupil_angle',
            timestamps=eye_timestamps,
            data=phi,
            unit='pixel',
            description='Features extracted from the video of the right eye.',
            comments='The rotation of the pupil ellipse in the video '
                    'frame. This is not registered to degrees visual angle, but '
                    'could be used to detect saccades or other changes in eye position.'
            )

            #add the ellipse fits
            pupil_track.add_timeseries(center_xy)
            pupil_track.add_timeseries(width_)
            pupil_track.add_timeseries(height_)
            pupil_track.add_timeseries(phi_)
            
            #close up the pupil_tracking module
            behavior_module.add_data_interface(pupil_track)

            nwb_path = os.path.join('/Users/danieldenman/Desktop/data/withCenter',os.path.basename(recording_folder))+'_c.nwb'
            with NWBHDF5IO(nwb_path, 'w') as io:
                io.write(nwbfile)
            print('SUCCEEDED making NWB for '+recording_folder)
        except: print('FAILED making NWB for '+dirName); return None
    return None

#walk folder tree
rootDir = '/Volumes/s1/OKR/sessions'

for dirName, subdirList, fileList in os.walk(rootDir):
    print('Found directory: %s' % dirName)
    if len(glob.glob(os.path.join(dirName,'*resnet*'))) > 2:
        if 'Cohort2' in dirName and 'Prelim' not in dirName:
            try:
                make_OKR_NWB(dirName) #do the work
            except: print('FAILED making NWB for '+dirName)
        else: print('NOT MAKING for '+dirName)

# Parallel(n_jobs=7)(make_OKR_NWB(dirName) for dirName, subdirList, fileList in os.walk(rootDir)) #do the work)
# make_OKR_NWB('/Volumes/s1/OKR/sessions/Cohort1/Week1Data/3582(wk1.1of2)/3582(wk1.1of2)_2021_8_17-19_24_41')